CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C        A Monte Carlo Generator for the Production and Decay of      C
C        Heavy Majorana Neutrinos in Electron-Proton and Positron-    C
C        Proton Collisions via Mixing between Light and Heavy         C
C        Majorana Neutrinos or via exchange of light righthanded      C
C        W-bosons                                                     C
C                                                                     C
C                           M A J O R                                 C
C                                                                     C
C                  Version 1.5.0 , 1996-02-16                         C
C                                                                     C
C        Authors: Johan Rathsman(a) and Gunnar Ingelman(a,b)          C
C                                                                     C
C             (a) Dept of Radiation Sciences                          C
C                 Uppsala University                                  C
C                 BOX 535                                             C
C                 S-751 21 UPPSALA, SWEDEN                            C
C                                                                     C
C             (b) DESY                                                C
C                 Notkestrasse 85                                     C
C                 D-22603 HAMBURG, FRG                                C
C                                                                     C
C        E-mail: RATHSMAN@TSL.UU.SE                                   C
C                INGELMAN@DESY.DE                                     C
C                                                                     C
C        Documentation: In preparation                                C
C                                                                     C
C        Please report any errors or suggestions for improvements.    C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      BLOCK DATA MADATA

      DOUBLE PRECISION DTRIES,DFCNSM

C--       Constants and default-values
      COMMON /MACONS/ PI,FCONST,BR1,BR2,BR3
      COMMON /MACROS/ DTRIES(3),DFCNSM(3),TCROSS(3),PCROSS(-6:6),NCALLS
      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)

      DATA PI/3.1415927/
      DATA DTRIES,DFCNSM,TCROSS,PCROSS,NCALLS/6*0.D0,16*0.,0/
      DATA MAFLAG/9,2*0,2*1,2*0,2,0,1,2,0,1,0,2,2*0,1,2*0/
      DATA PARM/100.,820.,30.,1000.,.01,1.E-3,1.1,23*0./
      DATA CUTM/0.,1.,0.,1.,4.,1.E8,9.,1.E8,0.,1.E3,-1.E2,1.E2/
      END

      SUBROUTINE MAINIT

C--       Initiates constants and starting values. Calculates the maxima
C--       of the g-functions if not given by the user.
C--       Has to be called once before Major is called.

C--       Called by: User

C--       Calls to: MADMAX, MADEFM, LINIT

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM
      REAL TCROSS,PCROSS
      DOUBLE PRECISION DTRIES,DFCNSM
      REAL PI,FCONST,BR1,BR2,BR3
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      REAL XMIN,XMAX,YMIN,YMAX,PLTMIN,PLTMAX
      REAL GMAX,ALS,ARS,ALSCUT,AX,AY,APLT
      REAL PARU,PARJ,PMAS,PARF,VCKM,BRAT,KFDP
      REAL CUT,PARL,GX,GY,GW2,GQ2,GU,PYPAR,PYVAR
      INTEGER MAFLAG,NCALLS,NPROC,NGIND
      INTEGER MSTU,MSTJ,KCHG,MDCY,MDME,LST,LST3,IPY
      CHARACTER CHAF*8

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)
      COMMON /MACROS/ DTRIES(3),DFCNSM(3),TCROSS(3),PCROSS(-6:6),NCALLS
      COMMON /MACONS/ PI,FCONST,BR1,BR2,BR3
      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /MAMAMI/ XMAX,XMIN,YMAX,YMIN,PLTMAX,PLTMIN
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /MAGSPE/ GMAX(3,2),ALS,ARS,ALSCUT,AX,AY,APLT,NGIND,NPROC
      COMMON /LUDAT1/ MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      COMMON /LUDAT2/ KCHG(500,3),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /LUDAT3/ MDCY(500,3),MDME(2000,2),BRAT(2000),KFDP(2000,5)
      COMMON /LUDAT4/ CHAF(500)
      COMMON /LEPTOU/ CUT(14),LST(40),PARL(30),GX,GY,GW2,GQ2,GU
      COMMON /LPFLAG/ LST3
      COMMON /LYPARA/ IPY(80),PYPAR(80),PYVAR(80)

C--       functions used
      INTEGER LUCOMP

C--        local variables
      REAL GF,ROTS,GAMMAW,GAMMAZ,GWL

      WRITE(*,1000)

C--        initiate constants and apropriate starting-values
      AMN=PARM(1)
      EP=SQRT(PARM(2)**2+PMAS(LUCOMP(2212),1)**2)
      EE=SQRT(PARM(3)**2+PMAS(LUCOMP(11),1)**2)

C--       initiate LEPTO
      LST(1)=2
      LST(2)=3
      LST(3)=2
      IF (MAFLAG(8).NE.1) THEN
         LST(8)=MAFLAG(8)
      ELSE
         LST(8)=0
      ENDIF
      LST(10)=0
      LST(15)=MAFLAG(1)
      PYPAR(12)=1.
C--       proton along positive z-axis
C--       check if incoming electron or positron
      IF (MAFLAG(7).GE.10) THEN
         CALL LINIT(0,-11,-ABS(PARM(3)),ABS(PARM(2)),2)
      ELSE
         CALL LINIT(0,11,-ABS(PARM(3)),ABS(PARM(2)),2)
      ENDIF
      LST3=MAFLAG(6)
      LST(22)=1
      LST(24)=1
      LST(28)=3
      LST(30)=0
      LST(31)=2

C--       no header from JETSET/PYTHIA
      MSTU(12)=0

C--       calculate constants and limits
      S=2.*EP*EE+2*ABS(PARM(2)*PARM(3))
     +  +PMAS(LUCOMP(2212),1)**2+PMAS(LUCOMP(11),1)**2
      ROTS=SQRT(S)
      WRITE(*,1090) ' CMS-energy: ',ROTS,' GeV'
      WRITE(*,1040) ' Parameter values: ',PARM
      WRITE(*,1030) ' Flags: ',MAFLAG
      WRITE(*,1050) ' Applied cuts:     ',CUTM
      AMN2=AMN*AMN
      XMIN=AMN2/S
      XMAX=1.
      YMIN=0.
      YMAX=1.-XMIN
      PLTMIN=0.
      ALSCUT=PARM(6)
      GF=PARL(17)
      GWL=GF*PMAS(24,1)**2
      AMZ=PMAS(23,1)
      AMZ2=AMZ**2
C--       set maximum spread of Z mass
      PMAS(23,3)=MIN(10.,AMN-AMZ-1.)
C--       check if right-handed current
      IF (MAFLAG(20).EQ.0) THEN
         AMW=PMAS(24,1)
         AMW2=AMW*AMW
         GAMMAW=(AMN2-AMW2)**2*(AMN2+2*AMW2)
         IF (AMN.LT.AMW) THEN
            GAMMAW=0.
            WRITE(*,*) 'Neutrino mass less than W-mass, program stopped'
            STOP
         ENDIF
         GAMMAZ=(AMN2-AMZ2)**2*(AMN2+2*AMZ2)
         IF (AMN.LT.AMZ) GAMMAZ=0.
C--       set maximum spread of W mass
         PMAS(24,3)=MIN(10.,AMN-AMW-1.)
      ELSE
         AMW=PARM(18)
         AMW2=AMW**2
C--       define W_R
         PMAS(34,1)=PARM(18)
         PMAS(34,2)=PARM(19)
C--       set maximum spread of W mass
         PMAS(34,3)=MIN(10.,AMN-AMW-1.)
         GAMMAW=(AMN2-AMW2)**2*(AMN2+2*AMW2)
         IF (AMN.LT.AMW) THEN
            GAMMAW=0.
            WRITE(*,*) 'Neutrino mass less than W-mass, program stopped'
            STOP
         ENDIF
         GAMMAZ=0.
C--       set W_R branching ratios (into light quarks only)
C--       Brancing ratio into top depending on top-mass (Nachtman p 422)
         IF (AMW.GT.PARF(106)) THEN
            BRAT(MDCY(34,2)+2)=0.
            BRAT(MDCY(34,2)+6)=0.00039*(1.-PARF(106)**2/AMW**2)*
     *(1.-0.5*PARF(106)**2/AMW**2-0.5*PARF(106)**4/AMW**4)
            BRAT(MDCY(34,2)+10)=0.249*(1.-PARF(106)**2/AMW**2)*
     *(1.-0.5*PARF(106)**2/AMW**2-0.5*PARF(106)**4/AMW**4)
            WRITE(*,*) 
            WRITE(*,*) 'Top branching ratio = ',BRAT(MDCY(34,2)+10)
         ELSE
            BRAT(MDCY(34,2)+2)=0.0
            BRAT(MDCY(34,2)+6)=0.0
            BRAT(MDCY(34,2)+10)=0.0
         ENDIF         
         BRAT(MDCY(34,2)+16)=0.0
         BRAT(MDCY(34,2)+17)=0.0
         BRAT(MDCY(34,2)+18)=0.0
C--       check entry point for W_R
         IF (MDCY(34,2).NE.231) THEN 
            WRITE(*,1110) 
         ENDIF
      ENDIF
      FCONST=GWL**2*PARM(5)*AMW**2*AMN2/(4*PI*PI*S)/
     / (2.*GAMMAW+GAMMAZ)

C--       calculate branching ratios
      BR1=GAMMAW/(2.*GAMMAW+GAMMAZ)
      BR2=BR1
      BR3=(1.-2*BR1)

C--       choose/calculate constants in h-functions 
      IF (MAFLAG(12).EQ.1) THEN
         AX=PARM(8)
      ELSE
         AX=10.4-23.616*AMN2/S
      ENDIF
      IF (MAFLAG(14).EQ.1) THEN
         AY=PARM(9)
      ELSE
         AY=4.4+15.744*AMN2/S
      ENDIF

C--       write information about chosen channels
      IF (MAFLAG(7).EQ.1 .OR. MAFLAG(7).EQ.0) THEN
         WRITE(*,1070) ' Channel nr 1, e- p --> e+ W- X'
         WRITE(*,1100) BR1
         PLTMAX=0.5*SQRT(S)*(AMN2-AMW2)/AMN2
         IF (MAFLAG(16).EQ.1) THEN
            APLT=PARM(10)
         ELSE
            APLT=46.*AMN2/(AMN2-AMW2)**2/S**.25
         ENDIF
         WRITE(*,1060) ' Ranges from kinematics: ',XMIN,' <  X  < ',
     ,   XMAX,YMIN,' <  Y  < ',YMAX,PLTMIN,' < PLT < ',PLTMAX
         WRITE(*,1080) AX,AY,APLT
      ENDIF

      IF (MAFLAG(7).EQ.2 .OR. MAFLAG(7).EQ.0) THEN
         WRITE(*,1070) ' Channel nr 2, e- p --> e- W+ X'
         WRITE(*,1100) BR2
         PLTMAX=0.5*SQRT(S)*(AMN2-AMW2)/AMN2
         IF (MAFLAG(16).EQ.1) THEN
            APLT=PARM(10)
         ELSE
            APLT=46.*AMN2/(AMN2-AMW2)**2/S**.25
         ENDIF
         WRITE(*,1060) ' Ranges from kinematics: ',XMIN,' <  X  < ',
     ,   XMAX,YMIN,' <  Y  < ',YMAX,PLTMIN,' < PLT < ',PLTMAX
         WRITE(*,1080) AX,AY,APLT
      ENDIF

      IF (MAFLAG(7).EQ.3 .OR. MAFLAG(7).EQ.0) THEN
         WRITE(*,1070) ' Channel nr 3, e- p --> nu Z X'
         IF (MAFLAG(20).EQ.1) THEN
            WRITE(*,*) ' Set to zero'
         ELSE
            WRITE(*,1100) BR3
            PLTMAX=0.5*SQRT(S)*(AMN2-AMZ2)/AMN2
            IF (MAFLAG(17).EQ.1) THEN
               APLT=PARM(11)
            ELSE
               APLT=46.*AMN2/(AMN2-AMZ2)**2/S**.25
            ENDIF
            WRITE(*,1060) ' Ranges from kinematics: ',XMIN,' <  X  < ',
     ,   XMAX,YMIN,' <  Y  < ',YMAX,PLTMIN,' < PLT < ',PLTMAX
            WRITE(*,1080) AX,AY,APLT
         ENDIF
      ENDIF

      IF (MAFLAG(7).EQ.11 .OR. MAFLAG(7).EQ.10) THEN
         WRITE(*,1070) ' Channel nr 11, e+ p --> e- W+ X'
         WRITE(*,1100) BR1
         PLTMAX=0.5*SQRT(S)*(AMN2-AMW2)/AMN2
         IF (MAFLAG(16).EQ.1) THEN
            APLT=PARM(10)
         ELSE
            APLT=46.*AMN2/(AMN2-AMW2)**2/S**.25
         ENDIF
         WRITE(*,1060) ' Ranges from kinematics: ',XMIN,' <  X  < ',
     ,   XMAX,YMIN,' <  Y  < ',YMAX,PLTMIN,' < PLT < ',PLTMAX
         WRITE(*,1080) AX,AY,APLT
      ENDIF

      IF (MAFLAG(7).EQ.12 .OR. MAFLAG(7).EQ.10) THEN
         WRITE(*,1070) ' Channel nr 12, e+ p --> e+ W- X'
         WRITE(*,1100) BR2
         PLTMAX=0.5*SQRT(S)*(AMN2-AMW2)/AMN2
         IF (MAFLAG(16).EQ.1) THEN
            APLT=PARM(10)
         ELSE
            APLT=46.*AMN2/(AMN2-AMW2)**2/S**.25
         ENDIF
         WRITE(*,1060) ' Ranges from kinematics: ',XMIN,' <  X  < ',
     ,   XMAX,YMIN,' <  Y  < ',YMAX,PLTMIN,' < PLT < ',PLTMAX
         WRITE(*,1080) AX,AY,APLT
      ENDIF

      IF (MAFLAG(7).EQ.13 .OR. MAFLAG(7).EQ.10) THEN
         WRITE(*,1070) ' Channel nr 13, e+ p --> nu Z X'
         IF (MAFLAG(20).EQ.1) THEN
            WRITE(*,*) ' Set to zero'
         ELSE
            WRITE(*,1100) BR3
            PLTMAX=0.5*SQRT(S)*(AMN2-AMZ2)/AMN2
            IF (MAFLAG(17).EQ.1) THEN
               APLT=PARM(11)
            ELSE
               APLT=46.*AMN2/(AMN2-AMZ2)**2/S**.25
            ENDIF
            WRITE(*,1060) ' Ranges from kinematics: ',XMIN,' <  X  < ',
     ,   XMAX,YMIN,' <  Y  < ',YMAX,PLTMIN,' < PLT < ',PLTMAX
            WRITE(*,1080) AX,AY,APLT
         ENDIF
      ENDIF

C--       adjust min and max to avoid numerical problems
      XMAX=0.999999
      YMIN=1.E-6
      PLTMIN=1.E-6

C--       calculate max of g-function if not given by user
      IF (MAFLAG(19).EQ.0.) THEN
         CALL MADMAX
      ELSE
         GMAX(1,1)=PARM(12)
         GMAX(1,2)=PARM(13)
         GMAX(2,1)=PARM(14)
         GMAX(2,2)=PARM(15)
         GMAX(3,1)=PARM(16)
         GMAX(3,2)=PARM(17)
      ENDIF

C--       shut off the top-decay channel for W
      MDME(174,1)=0

C--       shut off the top-decay channel for Z0
      MDME(153,1)=0

 
C--       define the majorana neutrino in JETSET 7.4 code.
      CALL MADEFM
     
 1000 FORMAT(/,
     &'          M       M     A A     JJJJJJJ    OOOOO    RRRRR',/,
     &'          M M   M M    A   A          J   O     O   R    R',/,
     &'          M   M   M   A     A         J   O     O   R    R',/,
     &'          M       M   AAAAAAA         J   O     O   RRRRR',/,
     &'          M       M   A     A         J   O     O   R  R',/,
     &'          M       M   A     A   J     J   O     O   R   R',/,
     &'          M       M   A     A    JJJJJ     OOOOO    R    R',//,
     &'       A   M O N T E   C A R L O   G E N E R A T O R   F O R',/,
     &'          H E A V Y   M A J O R A N A   N E U T R I N O S',/,
     &'          V E R S I O N   1 . 5 . 0   1 9 9 6 - 0 2 - 1 6 ',/)

1030  FORMAT(/,A,/T5, ' MAFLAG(1)=',I3,T25,' MAFLAG(2)=',I3,/T5,
     &              ' MAFLAG(3)=',I3,T25,' MAFLAG(4)=',I3,/T5,
     &              ' MAFLAG(5)=',I3,T25,' MAFLAG(6)=',I3,/T5,
     &              ' MAFLAG(7)=',I3,T25,' MAFLAG(8)=',I3,/T5,
     &              ' MAFLAG(9)=',I3,T25,' MAFLAG(10)=',I3,/T5,
     &              ' MAFLAG(11)=',I3,T25,' MAFLAG(12)=',I3,/T5,
     &              ' MAFLAG(13)=',I3,T25,' MAFLAG(14)=',I3,/T5,
     &              ' MAFLAG(15)=',I3,T25,' MAFLAG(16)=',I3,/T5,
     &              ' MAFLAG(17)=',I3,T25,' MAFLAG(18)=',I3,/T5,
     &              ' MAFLAG(19)=',I3,T25,' MAFLAG(20)=',I3)
1040  FORMAT(/,A,/T5 ' PARM(1)=',G10.4,T30,' PARM(2)=',G10.4,T55,
     &              ' PARM(3)=',G10.4,/T5,' PARM(4)=',G10.4,T30,
     &              ' PARM(5)=',G10.4,T55,' PARM(6)=',G10.4,/T5,
     &              ' PARM(7)=',G10.4,T30,' PARM(8)=',G10.4,T55,
     &              ' PARM(9)=',G10.4,/T5,' PARM(10)=',G10.4,T30,
     &              ' PARM(11)=',G10.4,T55,' PARM(12)=',G10.4,/T5,
     &              ' PARM(13)=',G10.4,T30,' PARM(14)=',G10.4,T55,
     &              ' PARM(15)=',G10.4,/T5,' PARM(16)=',G10.4,T30,
     &              ' PARM(17)=',G10.4,T55,' PARM(18)=',G10.4,/T5,
     &              ' PARM(19)=',G10.4,T30,' PARM(20)=',G10.4,T55,
     &              ' PARM(21)=',G10.4,/T5,' PARM(22)=',G10.4,T30,
     &              ' PARM(23)=',G10.4,T55,' PARM(24)=',G10.4,/T5,
     &              ' PARM(25)=',G10.4,T30,' PARM(26)=',G10.4,T55,
     &              ' PARM(27)=',G10.4,/T5,' PARM(28)=',G10.4,T30,
     &              ' PARM(29)=',G10.4,T55,' PARM(30)=',G10.4)
1050  FORMAT(/,A, G12.4,' <  X  < ',G12.4,/T20,
     &              G12.4,' <  Y  < ',G12.4,/T20,
     &              G12.4,' <  Q2 < ',G12.4,/T20,
     &              G12.4,' <  W2 < ',G12.4,/T20,
     &              G12.4,' < PLT < ',G12.4,/T20,
     &              G12.4,' <  YL < ',G12.4)
1060  FORMAT(A, G12.4,A,G12.4,/T26,G12.4,A,G12.4,/T26,
     &               G12.4,A,G12.4)
1070  FORMAT(/,A)
1080  FORMAT(' Constants in h-functons:  AX=',G12.4/T28,
     &       'AY=',G12.4,/T26,'APLT=',G12.4)
1090  FORMAT(/A, F7.2,A)
1100  FORMAT(' Branching ratio: ',F7.5)
1110  FORMAT(/,' WARNING! Possible error in JETSET interface.',/,
     & ' Check in the decay list generated by a call to LULIST(12)',/,
     & ' (after the program has executed) that the decay channels',/,
     & ' for the W_R (KF=34) are correct or conslut the author.',/,
     & ' (The leptonic decay channels should be zero).')

      END

      SUBROUTINE MAJOR

C--       Administers the generation of an event and fills the event record.

C--       Called by: User

C--       Calls to: MAGENE, MAERRM, MAFLAV, LUROBO, LUDBRB,
C--                 LSHOWR, LUSHOW, LUEXEC, LUPREP, LUEDIT

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM,TCROSS,PCROSS
      DOUBLE PRECISION DTRIES,DFCNSM
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      REAL XMIN,XMAX,YMIN,YMAX,PLTMIN,PLTMAX
      REAL PI,FCONST,BR1,BR2,BR3
      REAL GMAX,ALS,ARS,ALSCUT,AX,AY,APLT
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      REAL P,V,PARU,PARJ,PMAS,PARF,VCKM,BRAT
      REAL SPHI,STHETA,PB,PHIR
      DOUBLE PRECISION DBETA
      REAL CUT,PARL,GX,GY,GW2,GQ2,GU
      INTEGER MAFLAG,NCALLS,NGIND,NPROC
      INTEGER N,K,MSTU,MSTJ,KCHG,MDCY,MDME,KFDP,LST

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)
      COMMON /MACROS/ DTRIES(3),DFCNSM(3),TCROSS(3),PCROSS(-6:6),NCALLS
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /MAMAMI/ XMAX,XMIN,YMAX,YMIN,PLTMAX,PLTMIN
      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /MACONS/ PI,FCONST,BR1,BR2,BR3
      COMMON /MAGSPE/ GMAX(3,2),ALS,ARS,ALSCUT,AX,AY,APLT,NGIND,NPROC
      COMMON /LUJETS/ N,K(4000,5),P(4000,5),V(4000,5)
      COMMON /LUDAT1/ MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      COMMON /LUDAT2/ KCHG(500,3),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /LUDAT3/ MDCY(500,3),MDME(2000,2),BRAT(2000),KFDP(2000,5)
      COMMON /LBOOST/ DBETA(2,3),STHETA(2),SPHI(2),PB(5),PHIR
      COMMON /LEPTOU/ CUT(14),LST(40),PARL(30),GX,GY,GW2,GQ2,GU

C--       local variables
      REAL CPHI,PLL,EL,AML2
      INTEGER NINITQ,NSCATQ,I,J

C--       functions used
      INTEGER LUCOMP
      REAL ULMASS,RLU,ULANGL

C--       number of calls from user
      NCALLS=NCALLS+1

C--       simulate a phase space point according to the diff. cross-section
1     CALL MAGENE

C--       calculate the four-vectors from x,y,plt,yl and
C--       load the four-vectors into the event-record /LUJETS/

C--       reset event-record
      DO 10 I=1,10
         DO 20 J=1,5
            K(I,J)=0
            P(I,J)=0.
            V(I,J)=0.
20       CONTINUE
10    CONTINUE

C--       set incoming electron/positron in labframe
      K(1,1)=21
      IF (MAFLAG(7).GE.10) THEN
         K(1,2)=-11
      ELSE
         K(1,2)=11
      ENDIF
      P(1,3)=-ABS(PARM(3))
      P(1,4)=EE
      P(1,5)=PMAS(LUCOMP(K(1,2)),1)

C--       set incoming proton in labframe
      K(2,1)=21
      K(2,2)=2212
      P(2,3)=ABS(PARM(2))
      P(2,4)=EP
      P(2,5)=PMAS(LUCOMP(K(2,2)),1)

C--       set exchanged W-boson in labframe
      K(3,1)=21
      IF (MAFLAG(7).GE.10) THEN
         IF (MAFLAG(20).EQ.1) THEN
            K(3,2)=34
         ELSE
            K(3,2)=24
         ENDIF
      ELSE
         IF (MAFLAG(20).EQ.1) THEN
            K(3,2)=-34
         ELSE
            K(3,2)=-24
         ENDIF
      ENDIF
      K(3,3)=1
      P(3,1)=-PNT
      P(3,3)=P(1,3)-PNL
      P(3,4)=EE-EN
      P(3,5)=-SQRT(Q2)

C--       recalculate the hadronic cms-energy
      W2=(P(2,4)+P(3,4))**2-(P(2,1)+P(3,1))**2
     -  -(P(2,2)+P(3,2))**2-(P(2,3)+P(3,3))**2

C--       set majorana neutrino in labframe
      K(4,1)=1
      K(4,2)=79
      K(4,3)=1
      P(4,1)=PNT
      P(4,3)=PNL
      P(4,4)=EN
      P(4,5)=AMN

C--       calculate LEPTO-variables
      GX=X
      GY=Y
      GW2=W2
      GQ2=Q2
      GU=(P(2,1)*P(3,1)-P(2,2)*P(3,2)-P(2,3)*P(3,3)-P(2,4)*P(3,4))/
     /   P(2,5)

C--       choose flavour of initial and scattered quark
      CALL MAFLAV(NINITQ,NSCATQ)

C--       set initial quark in labframe
      LST(25)=NINITQ
      K(7,1)=21
      K(7,2)=NINITQ
      K(7,3)=2
      P(7,5)=ULMASS(K(7,2))
      P(7,4)=.5*(X*(P(2,4)+P(2,3))+P(7,5)**2/(X*(P(2,4)+P(2,3))))
      P(7,3)=.5*(X*(P(2,4)+P(2,3))-P(7,5)**2/(X*(P(2,4)+P(2,3))))

      IF (MAFLAG(8).NE.1) THEN
C--       initial and final state radiation

C--       set scattered quark in labframe
         K(5,1)=3
         K(5,2)=NSCATQ
         K(5,3)=2
         K(5,4)=60000
         K(5,5)=60000
         P(5,1)=-PNT
         P(5,3)=P(7,3)-PNL+P(1,3)
         P(5,4)=P(7,4)-EN+EE
         P(5,5)=ULMASS(K(5,2))

C--       set di-quark in labframe
         K(6,1)=3
         IF (NINITQ.EQ.2) THEN
            IF (RLU(0).LT.PARL(4)) THEN
               K(6,2)=2101
            ELSE
               K(6,2)=2103
            ENDIF
         ELSEIF (NINITQ.EQ.1) THEN
            K(6,2)=2203
         ELSE
            K(6,2)=-NINITQ
         ENDIF
         K(6,3)=2
         K(6,4)=50000
         K(6,5)=50000
         P(6,5)=ULMASS(K(6,2))
         P(6,4)=.5*((1.-X)*(P(2,4)+P(2,3))+P(6,5)**2/(1.-X)/
     /             (P(2,4)+P(2,3)))
         P(6,3)=.5*((1.-X)*(P(2,4)+P(2,3))-P(6,5)**2/(1.-X)/
     /             (P(2,4)+P(2,3)))

         N=7

C--       transform to cms of incoming particles, lepton along +z-axis
         DO 70 J=1,3
            DBETA(1,J)=DBLE(P(1,J)+P(2,J))/DBLE(P(1,4)+P(2,4))
70       CONTINUE
         CALL LUDBRB(1,N,0.,0.,-DBETA(1,1),-DBETA(1,2),-DBETA(1,3))
         SPHI(1)=ULANGL(P(1,1),P(1,2))
         CALL LUDBRB(1,N,0.,-SPHI(1),0.D0,0.D0,0.D0)
         STHETA(1)=ULANGL(P(1,3),P(1,1))
         CALL LUDBRB(1,N,-STHETA(1),0.,0.D0,0.D0,0.D0)

C--       prepare for parton cascade
         CALL LSHOWR(0)

C--       boost the event to the hadronic cms
         DO 80 J=1,3
            DBETA(2,J)=DBLE(P(3,J)+P(2,J))/DBLE(P(3,4)+P(2,4))
80       CONTINUE
         CALL LUDBRB(1,N,0.,0.,-DBETA(2,1),-DBETA(2,2),-DBETA(2,3))
         SPHI(2)=ULANGL(P(3,1),P(3,2))
         CALL LUDBRB(1,N,0.,-SPHI(2),0.D0,0.D0,0.D0)
         STHETA(2)=ULANGL(P(3,3),P(3,1))
         CALL LUDBRB(1,N,-STHETA(2),0.,0.D0,0.D0,0.D0)

C--       save momentum of exchanged boson
         DO 90 J=1,5
            PB(J)=P(3,J)
90       CONTINUE
         LST(26)=5
         LST(27)=0

         N=6

C--       include parton cascade (+ remnant & kt)
         CALL LSHOWR(1)

C--       transform to the lab-system
         CALL LUDBRB(0,0,STHETA(2),0.,0.D0,0.D0,0.D0)
         CALL LUDBRB(0,0,0.,SPHI(2),0.D0,0.D0,0.D0)
         CALL LUDBRB(0,0,0.,0.,DBETA(2,1),DBETA(2,2),DBETA(2,3))
         CALL LUDBRB(0,0,STHETA(1),0.,0.D0,0.D0,0.D0)
         CALL LUDBRB(0,0,0.,SPHI(1),0.D0,0.D0,0.D0)
         CALL LUDBRB(0,0,0.,0.,DBETA(1,1),DBETA(1,2),DBETA(1,3))

      ELSE
C--       final state radiation from the scattered quark only, using a
C--       simplified treatment (LUSHOW is used on the quark and diquark
C--       system). Only valence quarks, i.e. u-quarks, are considered 
C--       in the proton.

C--       set scattered quark in labframe
         K(8,1)=3
         IF (MAFLAG(7).GE.10) THEN
            K(8,2)=2
         ELSE
            IF (RLU(0).LT..95) THEN
               K(8,2)=1
            ELSE
               K(8,2)=3
            ENDIF
         ENDIF
         K(8,3)=2
         K(8,4)=90000
         K(8,5)=90000
         P(8,1)=-PNT
         P(8,3)=P(7,3)-PNL+P(1,3)
         P(8,4)=P(7,4)-EN+EE
         P(8,5)=ULMASS(K(8,2))

C--       set di-quark in labframe
         K(9,1)=3
         IF (MAFLAG(7).GE.10) THEN
            K(9,2)=2203
         ELSE
            IF (RLU(0).LT.PARL(4)) THEN
               K(9,2)=2101
            ELSE
               K(9,2)=2103
            ENDIF
         ENDIF
         K(9,3)=2
         K(9,4)=80000
         K(9,5)=80000
         P(9,5)=ULMASS(K(6,2))
         P(9,4)=.5*((1.-X)*(P(2,4)+P(2,3))+P(9,5)**2/(1.-X)/
     /             (P(2,4)+P(2,3)))
         P(9,3)=.5*((1.-X)*(P(2,4)+P(2,3))-P(9,5)**2/(1.-X)/
     /             (P(2,4)+P(2,3)))

C--       make copies of initial and scattered quark
         DO 30 J=1,5
            P(5,J)=P(7,J)
            P(6,J)=P(7,J)
            P(7,J)=P(8,J)
30       CONTINUE
         K(5,1)=21
         K(6,1)=21
         K(7,1)=21
         IF (MAFLAG(7).GE.10) THEN
            K(5,2)=1
            K(6,2)=1
         ELSE
            K(5,2)=2
            K(6,2)=2
         ENDIF
         K(7,2)=K(8,2)
         K(5,3)=2
         K(6,3)=5
         K(7,3)=6

         N=9

C--       final state radiation, time like parton shower
         CALL LUSHOW(8,9,SQRT(W2))
         CALL LUPREP(0)
         CALL LUEDIT(14)
      ENDIF

C--       add N-decay products to the eventrecord
C--       inactivate the Majorana Neutrino
      K(4,1)=21
      K(4,4)=N+1
      K(4,5)=N+2
C--       add final-state positron/electron/neutrino in labframe
      K(N+1,1)=1
C--       inactivate for jetreconstruction
      IF (MAFLAG(3).EQ.1) K(N+1,1)=21
      IF (NPROC.EQ.1 .OR. NPROC.EQ.12) THEN
         K(N+1,2)=-11
      ELSEIF (NPROC.EQ.2 .OR. NPROC.EQ.11) THEN
         K(N+1,2)=11
      ELSE
         K(N+1,2)=12
      ENDIF
      K(N+1,3)=4
      K(N+1,4)=0
      K(N+1,5)=0
      P(N+1,5)=ULMASS(K(N+1,2))
      AML2=P(N+1,5)**2
      PLL=SQRT(AML2+PLT*PLT)*SINH(YL)
      EL=SQRT(AML2+PLT*PLT+PLL*PLL)
      CPHI=(2.*EN*EL-(AMN2-AMB2)-2.*PNL*PLL)/(2.*PNT*PLT)
      IF (CPHI.GE.1.) THEN
         CPHI=1. 
      ENDIF
      IF (CPHI.LE.-1.) THEN 
         CPHI=-1.
      ENDIF
      P(N+1,1)=PLT*CPHI
      P(N+1,2)=PLT*SQRT(1.-CPHI*CPHI)
      P(N+1,3)=PLL
      P(N+1,4)=EL

C--       add on shell boson in labframe
      K(N+2,1)=1
      IF (NPROC.EQ.1 .OR. NPROC.EQ.12) THEN
         IF (MAFLAG(20).EQ.1) THEN
            K(N+2,2)=-34
         ELSE
            K(N+2,2)=-24
         ENDIF
      ELSEIF (NPROC.EQ.2 .OR. NPROC.EQ.11) THEN
         IF (MAFLAG(20).EQ.1) THEN
            K(N+2,2)=34
        ELSE
            K(N+2,2)=24
         ENDIF
      ELSE
         K(N+2,2)=23
      ENDIF
      K(N+2,3)=4
      K(N+2,4)=0
      K(N+2,5)=0
      P(N+2,1)=PNT-P(N+1,1)
      P(N+2,2)=-P(N+1,2)
      P(N+2,3)=PNL-PLL
      P(N+2,4)=EN-EL
      P(N+2,5)=AMB
      N=N+2


C--       rotate the event in the lab frame around the z-axis
      LST(29)=0
      IF (MAFLAG(4).EQ.1) THEN
         PHIR=2*PI*RLU(0)
         CALL LUROBO(0.,PHIR,0.,0.,0.)
         LST(29)=1
      ENDIF

C--       reset V-array
      DO 500 I=1,N
      DO 500 J=1,5
         V(I,J)=0
500   CONTINUE

      IF (LST(21).NE.0) THEN
C         WRITE(*,1010) 'Error in LEPTO subroutine indicated by LST(21)='
C     ,                 ,LST(21),' New event generated'
C 1010    FORMAT(A)
         LST(21)=0
C--       on error make a new event
         GOTO 1
      ENDIF

C--       hadronize
      IF (MAFLAG(5).EQ.1) CALL LUEXEC

      IF (MSTU(22).LT.INT(NCALLS*1.E-3)) MSTU(22)=MSTU(22)+1

C--       on error make a new event
      IF (MSTU(24).NE.0) GOTO 1

C--       update total cross-section
      PARL(24)=PARM(21)+PARM(23)+PARM(25)

      RETURN

      END

      SUBROUTINE MADEFM

C--       Defines the majorana neutrino in JETSET 7.4 code.

C--       Called by: MAINIT

C--       Calls to: -

      REAL PMAS,PARF,VCKM,BRAT
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      INTEGER KCHG,MDCY,MDME,KFDP,KC
      CHARACTER CHAF*8

      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /LUDAT2/ KCHG(500,3),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /LUDAT3/ MDCY(500,3),MDME(2000,2),BRAT(2000),KFDP(2000,5)
      COMMON /LUDAT4/ CHAF(500)

C--       compressed code
      KC=79
C--       name
      CHAF(KC)='nu_major'
C--       charge
      KCHG(KC,1)=0
C--       colour information
      KCHG(KC,2)=0
C--       the particle is its own antiparticle
      KCHG(KC,3)=0
C--       mass
      PMAS(KC,1)=AMN
C--       width
      PMAS(KC,2)=0.
      PMAS(KC,3)=0.
C--       average lifetime
      PMAS(KC,4)=0.
C--       Majorana neutrino as stable particle
      MDCY(KC,1)=0.
C--       entry point into the decay channel
      MDCY(KC,2)=0.
C--       total number of decay channels
      MDCY(KC,3)=0.

      RETURN

      END

      SUBROUTINE MADMAX

C--       Sets appropriate starting values for MINUIT and calls MINUIT for
C--       the calculation of the maxima of the g-function

C--       Called by: MAINIT

C--       Calls to: MAINEW

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      REAL XMIN,XMAX,YMIN,YMAX,PLTMIN,PLTMAX
      REAL GMAX,ALS,ARS,ALSCUT,AX,AY,APLT
      REAL XKIN(4),UKIN(4),WKIN(4),AIN(4),BIN(4),MAXFIN,RELUP,RELERR,
     &RELER2,FCNMAX
      INTEGER MAFLAG,NPROC,NGIND,LST3,N
      CHARACTER*10 NAMKIN,NAM

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)
      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /MAMAMI/ XMAX,XMIN,YMAX,YMIN,PLTMAX,PLTMIN
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /MAGSPE/ GMAX(3,2),ALS,ARS,ALSCUT,AX,AY,APLT,NGIND,NPROC
      COMMON /MAINUI/ XKIN,UKIN,WKIN,AIN,BIN,MAXFIN,RELUP,RELERR,
     &RELER2,FCNMAX
      COMMON /MAINUC/ NAMKIN(4),NAM(30)
      COMMON /LPFLAG/ LST3

C--       local variables
      REAL X1(3),Y1(3),PLT1(3),YL1(3),X2(3),Y2(3),PLT2(3),YL2(3)


C--       calculate maximum of g-function
C--       starting values for MINUIT , 1=x, 2=y
C--       UKIN=first try, WKIN=step, AIN=min, BIN=max
C--       transfered to MINUIT via common /MAINUI/

      NAMKIN(1)='         x'
      NAMKIN(2)='         y'
      WRITE(*,1010) ' Maxima found by MINUIT:'
      DO 10 N=1,3
C--       starting values for g1
         X=0.6*(XMAX-XMIN)+XMIN
         UKIN(1)=X
         WKIN(1)=0.5*X
         AIN(1)=XMIN
         BIN(1)=XMAX
         SHAT=X*S
         Y=0.5*(1.-AMN2/SHAT)
         UKIN(2)=Y
         WKIN(2)=0.3
         AIN(2)=0.
         BIN(2)=1.-XMIN
         NGIND=1
         IF (MAFLAG(7).GE.10) THEN
            NPROC=N+10
         ELSE
            NPROC=N
         ENDIF
         IF (N.EQ.1.OR.N.EQ.2) THEN
            AMB=AMW
            AMB2=AMW2
         ELSE
            AMB=AMZ
            AMB2=AMZ2
            IF(MAFLAG(20).EQ.1 .OR. AMN.LT.AMZ) GOTO 10
         ENDIF
         PLTMAX=0.5*SQRT(S)*(AMN2-AMB2)/AMN2
         IF (MAFLAG(16).EQ.1) THEN
            APLT=PARM(10)
         ELSEIF (MAFLAG(17).EQ.1) THEN
            APLT=PARM(11)
         ELSE
            APLT=46.*AMN2/(AMN2-AMB2)**2/S**.25
         ENDIF

         CALL MAINEW
         X1(N)=X
         Y1(N)=Y
         PLT1(N)=PLT
         YL1(N)=YL

C--       starting values for g2
         Y=1.-AMN2/SHAT
         UKIN(2)=Y
         NGIND=2
         CALL MAINEW
         X2(N)=X
         Y2(N)=Y
         PLT2(N)=PLT
         YL2(N)=YL
         WRITE(*,1020) ' GMAX(',N,',1)=',GMAX(N,1)
         WRITE(*,1030) ' in X=',X1(N),'  Y=',Y1(N),
     ,              '  PLT=',PLT1(N),'  YL=',YL1(N)
         WRITE(*,1020) ' GMAX(',N,',2)=',GMAX(N,2)
         WRITE(*,1030) ' in X=',X2(N),'  Y=',Y2(N),
     ,              '  PLT=',PLT2(N),'  YL=',YL2(N)
         GMAX(N,1)=MAX(GMAX(N,1),GMAX(N,2))
         GMAX(N,2)=GMAX(N,1)
10    CONTINUE

      GMAX(1,1)=MAX(GMAX(1,1),GMAX(2,1))
      GMAX(1,2)=GMAX(1,1)
      GMAX(2,1)=GMAX(1,1)
      GMAX(2,2)=GMAX(1,1)

      WRITE(*,1040) PARM(7)
1010  FORMAT(/,A)
1020  FORMAT(A, I1,A,G12.4)
1030  FORMAT(A, G12.4,A,G12.4,A,G12.4,A,G12.4)
1040  FORMAT(' where the maxima have been',
     &' multiplied with the safety factor:', G12.4,//)

      END

      SUBROUTINE MASIGX(NPAR,DERIV,DIFSIG,XF,NFLAG)

C--       Is called by MINUIT and is used as an interface to
C--       the g-function AMGFUN.
C--       Gives minus the g-function if within range and
C--       otherwise it gives the square of the distance to the limit.
C--       The search for the maxima is done along the singularity of the
C--       g-function (only x and y varies freely while plt and
C--       yl are calculated from x and y).
C--       Stores the maxima found by MINUIT in the vector GMAX.

C--       Called by: MINUIT-subroutines

C--       Calls to: MAERRM

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      REAL XMIN,XMAX,YMIN,YMAX,PLTMIN,PLTMAX
      REAL GMAX,ALS,ARS,ALSCUT,AX,AY,APLT
      INTEGER MAFLAG,NPROC,NGIND

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)
      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /MAMAMI/ XMAX,XMIN,YMAX,YMIN,PLTMAX,PLTMIN
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /MAGSPE/ GMAX(3,2),ALS,ARS,ALSCUT,AX,AY,APLT,NGIND,NPROC

C--       functions used
      REAL AMGFUN
C--       local variables
      REAL TEMP(1:10),C
      INTEGER NPAR,NFLAG,NCLS,NLOC
      REAL DERIV(30),DIFSIG,XF(30),DIFSG1,DIFSG2,YL1,YL2

      NPAR=2
      
      NLOC=MOD(NPROC,10)

      IF (NFLAG.EQ.1) NCLS=0
      IF (NFLAG.EQ.2)
     & CALL MAERRM(', warning from MINUIT, Gradient asked for',0)

      NCLS=NCLS+1

      X=XF(1)
      Y=XF(2)

C--       check x-limits
      IF (X.LT.XMIN) THEN
         DIFSIG=(X-XMIN)**2
         RETURN
      ENDIF
      IF (X.GT.XMAX) THEN
         DIFSIG=(X-XMAX)**2
         RETURN
      ENDIF

C--       check y-limits
      SHAT=X*S
      YMAX=1.-AMN2/SHAT
      IF (Y.LE.YMIN) THEN
         DIFSIG=(Y-YMIN)**2
         RETURN
      ENDIF
      IF (Y.GE.YMAX) THEN
         DIFSIG=(Y-YMAX)**2
         RETURN
      ENDIF

C--       calculate plt
      TEMP(1)=(1.-Y)*Y*SHAT-Y*AMN2
      IF (TEMP(1).LT.0.) THEN
         DIFSIG=TEMP(1)**2
         RETURN
      ENDIF

      IF (NGIND.EQ.1) THEN
         PLT=.5*(AMN2-AMB2)/((1.-ALSCUT)*SQRT((1.-Y)*AMN2+(1.-Y)*Y*SHAT)
     +                       +SQRT((1.-Y)*Y*SHAT-Y*AMN2))
      ELSE
         PLT=.5*(AMN2-AMB2)/((1.+ALSCUT)*SQRT((1.-Y)*AMN2+(1.-Y)*Y*SHAT)
     +                       +SQRT((1.-Y)*Y*SHAT-Y*AMN2))
      ENDIF

C--       calculate yl and the g-function
      PNT=SQRT((1.-Y)*Y*SHAT-Y*AMN2)
      PNL=X*Y*EP-(1.-Y)*EE+0.25*AMN2/EE
      AMNT=SQRT(AMN2+PNT*PNT)
      EN=SQRT(AMNT*AMNT+PNL*PNL)
      YN=0.5*LOG((EN+PNL)/(EN-PNL))
      TEMP(1)=2.*PNT*PLT
      TEMP(2)=2.*AMNT*PLT
      ALS=(AMN2-AMB2-TEMP(1))/TEMP(2)
      ARS=(AMN2-AMB2+TEMP(1))/TEMP(2)
C--       the borderline is given by yl=yn in the first part of g-function
      IF (NGIND.EQ.1) THEN
         YL=YN
         C=-ARS
         DIFSIG=-AMGFUN(C)
      ENDIF
C--       the borderline is given by yl=yn+/-f(alscut) in the second
C--       part of the g-function. The g-function is calculated in both
C--       points and the largest value is chosen.
      IF (NGIND.EQ.2) THEN
         TEMP(1)=LOG(1.+ALSCUT+SQRT(2*ALSCUT+ALSCUT**2))
         YL1=YN+TEMP(1)
         YL2=YN-TEMP(1)
         C=-ARS*ALS
         YL=YL1
         DIFSG1=-AMGFUN(C)
         YL=YL2
         DIFSG2=-AMGFUN(C)
         DIFSIG=MIN(DIFSG1,DIFSG2)
      ENDIF

C--       the last call from MINUIT is in the maximumpoint
      IF (NFLAG.EQ.3) GMAX(NLOC,NGIND)=-PARM(7)*DIFSIG

      RETURN

      END

      SUBROUTINE MAGENE

C--       Generates a phase space point according to the diff. cross-section.

C--       Called by: MAJOR

C--       Calls to: 

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM
      REAL TCROSS,PCROSS
      DOUBLE PRECISION DTRIES,DFCNSM
      REAL PI,FCONST,BR1,BR2,BR3
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      REAL XMIN,XMAX,YMIN,YMAX,PLTMIN,PLTMAX
      REAL GMAX,ALS,ARS,ALSCUT,AX,AY,APLT
CHISTO      REAL BLANC
      INTEGER MAFLAG,NCALLS,NPROC,NGIND
CHISTO      INTEGER NHBOOK
      REAL PMAS,PARF,VCKM,BRAT
      INTEGER KCHG,MDCY,MDME,KFDP

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)
      COMMON /MACROS/ DTRIES(3),DFCNSM(3),TCROSS(3),PCROSS(-6:6),NCALLS
      COMMON /MACONS/ PI,FCONST,BR1,BR2,BR3
      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /MAGSPE/ GMAX(3,2),ALS,ARS,ALSCUT,AX,AY,APLT,NGIND,NPROC
      COMMON /MAMAMI/ XMAX,XMIN,YMAX,YMIN,PLTMAX,PLTMIN
CHISTO      PARAMETER(NHBOOK=100000)
CHISTO      COMMON /PAWC/ BLANC(NHBOOK)
      COMMON /LUDAT2/ KCHG(500,3),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /LUDAT3/ MDCY(500,3),MDME(2000,2),BRAT(2000),KFDP(2000,5)

C--       functions used
      REAL RLU,AMGFUN,ULMASS

C--       local variables
      REAL PNTMIN,AMNTMI,TEMP(1:10)
      REAL OMEGA,OMEGAU,OMEGAL,C
      INTEGER NCUT(3),NEVENT(3),NLOC
      DOUBLE PRECISION DG2SUM(3),DSURVI(3),DACCEP(3),DTEMP

      DATA DG2SUM,DSURVI,DACCEP,NCUT,NEVENT/9*0.D0,6*0/
      SAVE DG2SUM,DSURVI,DACCEP,NCUT,NEVENT

C--       start the generation

C--       choose decay channel for heavy Majorana neutrino
      IF (MAFLAG(7).EQ.0) THEN
         TEMP(1)=RLU(0)
         IF (TEMP(1).LT.BR1) THEN
            NPROC=1
         ELSEIF (TEMP(1).LT.(BR1+BR2)) THEN
            NPROC=2
         ELSE
            NPROC=3
         ENDIF
      ELSEIF (MAFLAG(7).EQ.10) THEN
         TEMP(1)=RLU(0)
         IF (TEMP(1).LT.BR1) THEN
            NPROC=11
         ELSEIF (TEMP(1).LT.(BR1+BR2)) THEN
            NPROC=12
         ELSE
            NPROC=13
         ENDIF
      ELSE
         NPROC=MAFLAG(7)
      ENDIF
      
      NLOC=MOD(NPROC,10)

      NEVENT(NLOC)=NEVENT(NLOC)+1

C--       fixed or varying boson masses
      IF (NLOC.EQ.1.OR.NLOC.EQ.2) THEN
         IF (MAFLAG(18).EQ.1) THEN
            IF (MAFLAG(20).EQ.1) THEN
               AMB=ULMASS(34)
C--       adjust  branching ratio into top
               IF (AMB.GT.PARF(106)) THEN
                  BRAT(MDCY(34,2)+10)=0.249*(1.-PARF(106)**2/AMB**2)*
     *(1.-0.5*PARF(106)**2/AMB**2-0.5*PARF(106)**4/AMB**4)
               ELSE
                  BRAT(MDCY(34,2)+10)=0.0
               ENDIF         
            ELSE
               AMB=ULMASS(24)
            ENDIF
         ELSE
            AMB=AMW
         ENDIF
      ELSE
         IF (MAFLAG(18).EQ.1) THEN
            AMB=ULMASS(23)
         ELSE
            AMB=AMZ
         ENDIF
      ENDIF
      AMB2=AMB**2

C--       recalculate plt-max and A-plt for chosen decay channel
      PLTMAX=0.5*SQRT(S)*(AMN2-AMB2)/AMN2
      IF (MAFLAG(16).EQ.1) THEN
         APLT=PARM(10)
      ELSEIF (MAFLAG(17).EQ.1) THEN
         APLT=PARM(11)
      ELSE
         APLT=46.*AMN2/(AMN2-AMB2)**2/S**.25
      ENDIF

10    CONTINUE

C--       choose plt according to chosen hplt-function
      IF (MAFLAG(15).EQ.-2) THEN
         PLT=PLTMAX*PLTMIN/(PLTMAX-RLU(0)*(PLTMAX-PLTMIN))
      ELSEIF (MAFLAG(15).EQ.-1) THEN
         PLT=PLTMIN*(PLTMAX/PLTMIN)**RLU(0)
      ELSEIF (MAFLAG(15).EQ.1) THEN
         PLT=-LOG(EXP(-APLT*PLTMIN)-RLU(0)*
     *      (EXP(-APLT*PLTMIN)-EXP(-APLT*PLTMAX)))/APLT
      ELSEIF (MAFLAG(15).EQ.2) THEN
         PLT=SQRT(-1./APLT*LOG(EXP(-PLTMIN*PLTMIN*APLT)-RLU(0)*
     *    (EXP(-PLTMIN*PLTMIN*APLT)-EXP(-PLTMAX*PLTMAX*APLT))))
      ELSE 
         PLT=PLTMIN+RLU(0)*(PLTMAX-PLTMIN)
      ENDIF
C--       check that plt>0
      IF (PLT.LE.0) GOTO 10

CHISTO      CALL HF1(1001,PLT,1.)

C--       choose x according to chosen hx-function
      TEMP(1)=(AMN2*PLT*PLT-0.25*(AMN2-AMB2)**2)/((AMN2-AMB2)*PLT)
      IF (TEMP(1).GT.0.0) THEN
         PNTMIN=TEMP(1)
      ELSE
         PNTMIN=0.0
      ENDIF
      AMNTMI=SQRT(AMN2+PNTMIN*PNTMIN)
      XMIN=(PNTMIN+AMNTMI)**2/S
      IF (MAFLAG(11).EQ.-2) THEN
         X=XMAX*XMIN/(XMAX-RLU(0)*(XMAX-XMIN))
      ELSEIF (MAFLAG(11).EQ.-1) THEN
         X=XMIN*(XMAX/XMIN)**RLU(0)
      ELSEIF (MAFLAG(11).EQ.1) THEN
         X=-LOG(EXP(-AX*XMIN)-RLU(0)*(EXP(-AX*XMIN)-EXP(-AX*XMAX)))/AX
      ELSEIF (MAFLAG(11).EQ.2) THEN
         X=SQRT(-1./AX*LOG(EXP(-AX*XMIN*XMIN)-RLU(0)*
     *     (EXP(-AX*XMIN*XMIN)-EXP(-AX*XMAX*XMAX))))
      ELSE 
         X=XMIN+RLU(0)*(XMAX-XMIN)
      ENDIF

CHISTO      CALL HF1(1002,X,1.)

C--       choose y according to chosen hy-function
      SHAT=X*S
      TEMP(2)=(SHAT-AMN2)**2-4.*SHAT*PNTMIN*PNTMIN
      IF (TEMP(2).LT.0) THEN
         TEMP(2)=0.
      ENDIF
      TEMP(1)= SQRT(TEMP(2))
      YMIN=0.5*(SHAT-AMN2-TEMP(1))/SHAT
      YMIN=MAX(1.E-6,YMIN)
      YMAX=0.5*(SHAT-AMN2+TEMP(1))/SHAT
      IF (MAFLAG(13).EQ.-2) THEN
         Y=YMAX*YMIN/(YMAX-RLU(0)*(YMAX-YMIN))
      ELSEIF (MAFLAG(13).EQ.-1) THEN
         Y=YMIN*(YMAX/YMIN)**RLU(0)
      ELSEIF (MAFLAG(13).EQ.1) THEN
         Y=-LOG(EXP(-AY*YMIN)-RLU(0)*(EXP(-AY*YMIN)-EXP(-AY*YMAX)))/AY
      ELSEIF (MAFLAG(13).EQ.2) THEN
         Y=SQRT(-1./AY*LOG(EXP(-AY*YMIN*YMIN)-RLU(0)*
     *     (EXP(-AY*YMIN*YMIN)-EXP(-AY*YMAX*YMAX))))
      ELSE 
         Y=YMIN+RLU(0)*(YMAX-YMIN)
      ENDIF

CHISTO      CALL HF1(1003,Y,1.)

C--         calculate transverse momentum of the heavy Majorana neutrino, pnt
      TEMP(3)=(1.-Y)*Y*SHAT-Y*AMN2
      IF (TEMP(3).LT.0.) THEN
         TEMP(3)=0.
      ENDIF
      PNT=SQRT(TEMP(3))
      PNL=X*Y*EP-(1.-Y)*EE+0.25*AMN2/EE
      AMNT=SQRT(AMN2+PNT*PNT)
      EN=SQRT(AMNT*AMNT+PNL*PNL)
      YN=0.5*LOG((EN+PNL)/(EN-PNL))

C--       calculate Q2
      Q2=2.*EE*(EN+PNL)-AMN2

C--         calculate the limits ALS and ARS for cosh(yn-yl)
      TEMP(1)=2.*PNT*PLT
      TEMP(2)=2.*AMNT*PLT
      ALS=(AMN2-AMB2-TEMP(1))/TEMP(2)
      ARS=(AMN2-AMB2+TEMP(1))/TEMP(2)
      OMEGAU=ARS
      IF (ALS.GE.1.) THEN
C--       choose g-function 2
         OMEGAL=ALS
         NGIND=2
      ELSE
C--       choose g-function 1
         OMEGAL=1
         NGIND=1
      ENDIF

C--       choose omega=cosh(yn-yl)
      C=-OMEGAL*OMEGAU
      OMEGA=-2.*OMEGAL*OMEGAU/
     /((OMEGAU-OMEGAL)*SIN(PI*RLU(0)-0.5*PI)-OMEGAL-OMEGAU)
      IF (OMEGA.LT.1.) OMEGA=1.

      IF (OMEGA.LT.(1.+ALSCUT) .AND.
     &    ALS.GT.(1.-ALSCUT) .AND. ALS.LT.(1.+ALSCUT)) THEN
C--       cut away
         NCUT(NLOC)=NCUT(NLOC)+1
         GOTO 10
      ENDIF

C--       calculate yl from omega
      TEMP(1)=LOG(OMEGA+SQRT(OMEGA*OMEGA-1.))
      TEMP(2)=RLU(0)
      IF (TEMP(2).GT.0.5) THEN
         YL=YN+TEMP(1)
      ELSE
         YL=YN-TEMP(1)
      ENDIF

CHISTO      CALL HF1(1004,YL,1.)

C--       calculate the g-function in the chosen point
      TEMP(1)=AMGFUN(C)

      DTRIES(NLOC)=DTRIES(NLOC)+1.D0
      DFCNSM(NLOC)=DFCNSM(NLOC)+TEMP(1)
      DG2SUM(NLOC)=DG2SUM(NLOC)+TEMP(1)**2
C--       update efficiency
      PARM(26+NLOC)=NEVENT(NLOC)/DTRIES(NLOC)

C--       check if the maximum of the g-function is violated
      IF (TEMP(1).GT.GMAX(NLOC,NGIND)) THEN
         MAFLAG(9)=1
         IF (GMAX(NLOC,NGIND).EQ.0.) THEN
            TEMP(2)=1.E9
         ELSE
            TEMP(2)=TEMP(1)/GMAX(NLOC,NGIND)
         ENDIF
         CALL MAERRM('. Maximum of g-function violated by factor',
     , TEMP(2))
         WRITE(*,1010) INT(DTRIES(NLOC)),NLOC,NGIND,
     ,   GMAX(NLOC,NGIND),NLOC,NGIND,TEMP(1),X,Y,PLT,YL
         GMAX(NLOC,NGIND)=TEMP(1)
      ENDIF

C--       "throw the arrow"
      IF (TEMP(1).GE.RLU(0)*GMAX(NLOC,NGIND)) THEN
C--       phase-space point accepted
         DACCEP(NLOC)=DACCEP(NLOC)+1.D0
C--       apply cuts
         W2=Q2*(1.-X)/X+.88035
         IF (X.LT.CUTM(1) .OR. X.GT.CUTM(2)) GOTO 10
         IF (Y.LT.CUTM(3) .OR. Y.GT.CUTM(4)) GOTO 10
         IF (Q2.LT.CUTM(5) .OR. Q2.GT.CUTM(6)) GOTO 10
         IF (W2.LT.CUTM(7) .OR. W2.GT.CUTM(8)) GOTO 10
         IF (PLT.LT.CUTM(9) .OR. PLT.GT.CUTM(10)) GOTO 10
         IF (YL.LT.CUTM(11) .OR. YL.GT.CUTM(12)) GOTO 10
C--       point survived cuts
         DSURVI(NLOC)=DSURVI(NLOC)+1.D0
C--       do an update of the approximation of the cross-section (in pb)
         TCROSS(NLOC)=DFCNSM(NLOC)/DTRIES(NLOC)
C--       correct for cuts
         PARM(19+2*NLOC)=TCROSS(NLOC)*DSURVI(NLOC)/DACCEP(NLOC)
         DTEMP=DG2SUM(NLOC)-DFCNSM(NLOC)**2/DTRIES(NLOC)
         IF (DTEMP.GE.0.D0) THEN
            PARM(20+2*NLOC)=SQRT(DTEMP)/DTRIES(NLOC)
         ELSE
            PARM(20+2*NLOC)=0.D0
CJR            WRITE(*,*) 'Sqrt of Neg :',DTEMP
         ENDIF
         RETURN
      ELSE
C--       phase-space point rejected, try a new point
         GOTO 10
      ENDIF

1010  FORMAT(/,
     & ' Warning, Maximum of the g-function has been exceeded after'
     & ,I9,' tries.'
     & ,/,' Old g-max(',I1,',',I1')=',G14.7
     &    ' New g-max(',I1,',',I1')=',G14.7
     & ,/,' In x=',G14.7,' y=',G14.7,' plt=',G14.7,' yl=',G14.7
     & ,/,' Maximum of g-function updated to new value.')

      END

      REAL FUNCTION AMGFUN(C)

C--       Calculates the value of the g-function in a given point

C--       Called by: MAGENE, MASIGX

C--       Calls to: LYSTFU

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM
      REAL TCROSS,PCROSS
      DOUBLE PRECISION DTRIES,DFCNSM
      REAL PI,FCONST,BR1,BR2,BR3
      REAL AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      REAL XMIN,XMAX,YMIN,YMAX,PLTMIN,PLTMAX
      REAL GMAX,ALS,ARS,ALSCUT
      REAL AX,AY,APLT,CUT,PARL,GX,GY,GW2,GQ2,GU
      REAL PMAS,PARF,VCKM
      INTEGER KCHG
      INTEGER MAFLAG,NCALLS,NPROC,NGIND,LST

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)
      COMMON /MACROS/ DTRIES(3),DFCNSM(3),TCROSS(3),PCROSS(-6:6),NCALLS
      COMMON /MACONS/ PI,FCONST,BR1,BR2,BR3
      COMMON /MAMASS/ AMB,AMB2,AMW,AMW2,AMN,AMN2,AMNT,AMZ,AMZ2
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /MAGSPE/ GMAX(3,2),ALS,ARS,ALSCUT,AX,AY,APLT,NGIND,NPROC
      COMMON /MAMAMI/ XMAX,XMIN,YMAX,YMIN,PLTMAX,PLTMIN
      COMMON /LEPTOU/ CUT(14),LST(40),PARL(30),GX,GY,GW2,GQ2,GU
      COMMON /LUDAT2/ KCHG(500,3),PMAS(500,4),PARF(2000),VCKM(4,4)

C--       local variables
      REAL C
      REAL SCALE,F,ANP,BNP,ANN,BNN,AN,BN
      REAL CPN,DPN,CPP,DPP,CP,DP
      REAL XPQ(-6:6)
      INTEGER KF
      
C--       choose particle with standard PDG-codes
      KF=2212

C--       choose scale for calculation of structure functions
      IF (MAFLAG(2).EQ.0) THEN
C--       default mu2=Q2
         SCALE=Q2
      ELSEIF (MAFLAG(2).EQ.1) THEN
         SCALE=PNT*PNT
      ELSEIF(MAFLAG(2).EQ.2) THEN
         SCALE=PARM(4)
      ENDIF

C--       calculate the structure functions
      CALL LYSTFU(KF,X,SCALE,XPQ)

C--       the cross-section according to B&G and R
      F=FCONST/(X*(Y*SHAT+AMW2)**2*AMNT)
      IF (NGIND.EQ.1) F=F/SQRT(COSH(YN-YL)-ALS)

      IF (NPROC.EQ.1) THEN
         ANP=4.*S*AMN2/AMB2*((SHAT-2.*AMB2)*(AMN2-AMB2)-
     -       2.*(AMN2-2*AMB2)*PLT*(EE*EXP(YL)+X*EP*EXP(-YL)))
         BNP=4.*(SHAT*(1.-Y)-AMN2)*AMN2/AMB2*
     *       (S*(1.-Y)*(AMN2-AMB2)-2.*(AMN2-2*AMB2)*EP*PLT*EXP(-YL))
         PCROSS(-1)=BNP*XPQ(-1)
         PCROSS(2)=ANP*XPQ(2)
         PCROSS(-3)=BNP*XPQ(-3)
         PCROSS(4)=ANP*XPQ(4)

      ELSEIF (NPROC.EQ.2) THEN
         ANN=8.*S*((AMN2-AMB2)*(SHAT-.5*AMN2**2/AMB2)-
     -       2.*(AMN2-.5*AMN2**2/AMB2)*PLT*(EE*EXP(YL)+X*EP*EXP(-YL)))
         BNN=8.*(SHAT*(1.-Y)-AMN2)*
     *       (S*(1.-Y)*(AMN2-AMB2)-
     -        2.*(AMN2-.5*AMN2**2/AMB2)*EP*PLT*EXP(-YL))
         PCROSS(-1)=BNN*XPQ(-1)
         PCROSS(2)=ANN*XPQ(2)
         PCROSS(-3)=BNN*XPQ(-3)
         PCROSS(4)=ANN*XPQ(4)

      ELSEIF (NPROC.EQ.3) THEN
C--       factor 1/(2*(cos(thetaweinberg))**2) taken into account when
C--       the Majorana neutrino decays into a Z-boson and a neutrino
         AN=4.*S*(SHAT-AMN2)*(AMN2-AMB2)*(AMN2+2.*AMB2)/AMB2
         AN=AN/(2.*PMAS(24,1)**2/AMZ2)
         BN=4.*S*(1.-Y)*(SHAT*(1.-Y)-AMN2)*(AMN2-AMB2)*
     *      (AMN2+2.*AMB2)/AMB2
         BN=BN/(2.*PMAS(24,1)**2/AMZ2)
         PCROSS(-1)=BN*XPQ(-1)
         PCROSS(2)=AN*XPQ(2)
         PCROSS(-3)=BN*XPQ(-3)
         PCROSS(4)=AN*XPQ(4)

      ELSEIF (NPROC.EQ.11) THEN
         DPN=4.*S*AMN2/AMB2*((SHAT-2.*AMB2)*(AMN2-AMB2)-
     -       2.*(AMN2-2*AMB2)*PLT*(EE*EXP(YL)+X*EP*EXP(-YL)))
         CPN=4.*(SHAT*(1.-Y)-AMN2)*AMN2/AMB2*
     *       (S*(1.-Y)*(AMN2-AMB2)-2.*(AMN2-2*AMB2)*EP*PLT*EXP(-YL))
         PCROSS(1)=CPN*XPQ(1)
         PCROSS(-2)=DPN*XPQ(-2)
         PCROSS(3)=CPN*XPQ(3)
         PCROSS(-4)=DPN*XPQ(-4)

      ELSEIF (NPROC.EQ.12) THEN
         DPP=8.*S*((AMN2-AMB2)*(SHAT-.5*AMN2**2/AMB2)-
     -       2.*(AMN2-.5*AMN2**2/AMB2)*PLT*(EE*EXP(YL)+X*EP*EXP(-YL)))
         CPP=8.*(SHAT*(1.-Y)-AMN2)*
     *       (S*(1.-Y)*(AMN2-AMB2)-
     -        2.*(AMN2-.5*AMN2**2/AMB2)*EP*PLT*EXP(-YL))
         PCROSS(1)=CPP*XPQ(1)
         PCROSS(-2)=DPP*XPQ(-2)
         PCROSS(3)=CPP*XPQ(3)
         PCROSS(-4)=DPP*XPQ(-4)

      ELSEIF (NPROC.EQ.13) THEN
C--       factor 1/(2*(cos(thetaweinberg))**2) taken into account when
C--       the Majorana neutrino decays into a Z-boson and a neutrino
         DP=4.*S*(SHAT-AMN2)*(AMN2-AMB2)*(AMN2+2.*AMB2)/AMB2
         DP=DP/(2.*PMAS(24,1)**2/AMZ2)
         CP=4.*S*(1.-Y)*(SHAT*(1.-Y)-AMN2)*(AMN2-AMB2)*
     *      (AMN2+2.*AMB2)/AMB2
         CP=CP/(2.*PMAS(24,1)**2/AMZ2)
         PCROSS(1)=CP*XPQ(1)
         PCROSS(-2)=DP*XPQ(-2)
         PCROSS(3)=CP*XPQ(3)
         PCROSS(-4)=DP*XPQ(-4)
      ENDIF
      IF (MAFLAG(7).GE.10) THEN
         PCROSS(0)=PCROSS(1)+PCROSS(-2)+PCROSS(3)+PCROSS(-4)
      ELSE
         PCROSS(0)=PCROSS(-1)+PCROSS(2)+PCROSS(-3)+PCROSS(4)
      ENDIF
      F=F*PCROSS(0)

C--      Multiply with integrals of the h-functions
C--      and divide with h-functions to get the g-function

      IF (MAFLAG(11).EQ.-2) THEN
         F=F*(1./XMIN-1./XMAX)
         F=F*X**2
      ELSEIF (MAFLAG(11).EQ.-1) THEN
         F=F*LOG(XMAX/XMIN)
         F=F*X
      ELSEIF (MAFLAG(11).EQ.1) THEN
         F=F*(EXP(-AX*XMIN)-EXP(-AX*XMAX))/AX
         F=F/EXP(-AX*X)
      ELSEIF (MAFLAG(11).EQ.2) THEN
         F=F*(EXP(-AX*XMIN*XMIN)-EXP(-AX*XMAX*XMAX))/(2.*AX)
         F=F/(X*EXP(-AX*X*X))
      ELSE 
         F=F*(XMAX-XMIN)
         F=F
      ENDIF

      IF (MAFLAG(13).EQ.-2) THEN
         F=F*(1./YMIN-1./YMAX)
         F=F*Y**2
      ELSEIF (MAFLAG(13).EQ.-1) THEN
         F=F*LOG(YMAX/YMIN)
         F=F*Y
      ELSEIF (MAFLAG(13).EQ.1) THEN
         F=F*(EXP(-AY*YMIN)-EXP(-AY*YMAX))/AY
         F=F/EXP(-AY*Y)
      ELSEIF (MAFLAG(13).EQ.2) THEN
         F=F*(EXP(-AY*YMIN*YMIN)-EXP(-AY*YMAX*YMAX))/(2.*AY)
         F=F/(Y*EXP(-AY*Y*Y))
      ELSE 
         F=F*(YMAX-YMIN)
         F=F
      ENDIF

      IF (MAFLAG(15).EQ.-2) THEN
         F=F*(1./PLTMIN-1./PLTMAX)
         F=F*PLT**2
      ELSEIF (MAFLAG(15).EQ.-1) THEN
         F=F*LOG(PLTMAX/PLTMIN)
         F=F*PLT
      ELSEIF (MAFLAG(15).EQ.1) THEN
         F=F*(EXP(-APLT*PLTMIN)-EXP(-APLT*PLTMAX))/APLT
         F=F/EXP(-APLT*PLT)
      ELSEIF (MAFLAG(15).EQ.2) THEN
         F=F*(EXP(-PLTMIN*PLTMIN*APLT)-EXP(-PLTMAX*PLTMAX*APLT))/
     /       (2.*APLT)
         F=F/(PLT*EXP(-PLT*PLT*APLT))
      ELSE 
         F=F*(PLTMAX-PLTMIN)
         F=F
      ENDIF

      F=F*2.*PI/SQRT(-C)
      IF (NGIND.EQ.1) THEN
         F=F*COSH(YN-YL)/SQRT(1.+COSH(YN-YL))
      ELSE
         F=F/ABS(TANH(YN-YL))
      ENDIF

      IF (F.LT.0) THEN
         F=0.
      ENDIF

C--       change units to pb instead of GeV^-2
      AMGFUN=3.894E8*F

      RETURN

      END

      SUBROUTINE MAFLAV(NINITQ,NSCATQ)

C--       Choose flavour of initial quark and scattered quark.

C--       Called by: MAJOR

C--       Calls to: -

C--       global variables
      IMPLICIT NONE
      REAL TCROSS,PCROSS
      DOUBLE PRECISION DTRIES,DFCNSM
      REAL CABIBO,PMAS,PARF,VCKM
      REAL EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL

      INTEGER NCALLS,KCHG

      COMMON /MACROS/ DTRIES(3),DFCNSM(3),TCROSS(3),PCROSS(-6:6),NCALLS
      COMMON /MAKINE/ EP,EE,S,SHAT,Q2,W2,X,Y,PNT,PNL,YN,EN,PLT,YL
      COMMON /LUDAT2/ KCHG(500,3),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /LFLMIX/ CABIBO(4,4)

C--       functions used
      REAL RLU
      INTEGER LUCOMP

C--       local variables
      REAL CHOICE,PCSUM,CABSUM
      INTEGER ID,IU,N,NA,NINITQ,NSCATQ,MINC,MOUT

C--       choose flavour of initial quark
100   CHOICE=RLU(0)*PCROSS(0)
      PCSUM=0.
      DO 10 N=-6,-1
         PCSUM=PCSUM+PCROSS(N)
         IF (CHOICE.LE.PCSUM) GOTO 30
10    CONTINUE
      DO 20 N=1,6
         PCSUM=PCSUM+PCROSS(N)
         IF (CHOICE.LT.PCSUM) GOTO 30
20    CONTINUE
30    NINITQ=N

C--       choose flavour of scattered quark
      NA=ABS(NINITQ)
      MINC=(NA+1)/2
      ID=MOD(NA,2)
      IU=MOD(NA+1,2)
      CHOICE=RLU(0)
      CABSUM=0.
      DO 40 MOUT=1,4
         CABSUM=CABSUM+CABIBO(IU*MINC+ID*MOUT,ID*MINC+IU*MOUT)
         IF(CHOICE.LT.CABSUM) GOTO 50
40    CONTINUE
C--       the scattered quark has same sign as the initial quark
50    NSCATQ=(2*MOUT-IU)*NINITQ/NA

C--       threshold function for heavy quarks
      IF (ABS(NINITQ).GE.4.OR.ABS(NSCATQ).GE.4) THEN
         IF (1.-(.938+PMAS(LUCOMP(NSCATQ),1)+PMAS(LUCOMP(NINITQ),1)
     +           +2*PMAS(1,1))**2/W2.LT.1.*RLU(0)) GOTO 100
      ENDIF

      RETURN

      END

      SUBROUTINE MAERRM(CHERRM,ERRVAL)

C--       Handle errors and write error messages.

C--       Called by: MAJOR, MASIGX, MAGENE, AMGFUN

C--       Calls to: -

C--       global variables
      IMPLICIT NONE
      REAL CUTM,PARM
      INTEGER MAFLAG

      COMMON /MAUSER/ MAFLAG(20),CUTM(12),PARM(30)

C--       local variables
      INTEGER NERROR
      REAL ERRVAL
      CHARACTER*42 CHERRM

      NERROR=MAFLAG(9)
      MAFLAG(9)=0

      IF (MAFLAG(10).EQ.0) THEN
         RETURN
      ELSEIF (MAFLAG(10).EQ.1) THEN
         WRITE(*,1010) NERROR,CHERRM,ERRVAL
         RETURN
      ELSE 
         WRITE(*,1020) NERROR,CHERRM,ERRVAL
         STOP
      ENDIF

1010  FORMAT(/,' Error nr',I3,A,G14.7,/)
1020  FORMAT(/,' Error nr',I3,A,G14.7,/,' Execution stopped')

      END

C--------------------------------------------------------------------
      BLOCK DATA MAMDAT

C...Give sensible default values to switches and parameters.
      COMMON /MAINUI/ XKIN(4),UKIN(4),WKIN(4),AIN(4),BIN(4),
     &MAXFIN,RELUP,RELERR,RELER2,FCNMAX
      COMMON /MAINUC/ NAMKIN(4),NAM(30)
      CHARACTER*10 NAMKIN,NAM

      DATA XKIN/1.,2.,3.,4./,UKIN,WKIN,AIN,BIN/16*0./,MAXFIN/2000/
      DATA RELUP,RELERR,RELER2/0.1,0.05,0.05/
      DATA NAMKIN/'         x','          ','          ','          '/

      END
      
      SUBROUTINE MACMND

C...This is the MINUIT routine COMAND.
CC        GETS IFORMATION FROM /MAINUI/  AND TAKES APPROPRIATE ACTION,
CC        EITHER DIRECTLY BY SKIPPING TO THE CORRESPONDING CODE IN
CC        MACMND, OR BY SETTING UP A CALL TO A SUBROUTINE
CC
      COMMON /MAINUI/ XKIN(4),UKIN(4),WKIN(4),AIN(4),BIN(4),
     &MAXFIN,RELUP,RELERR,RELER2,FCNMAX
      COMMON /LPFLAG/ LST3

      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC
      FVAL3 = 2.0*AMIN+1.0
C                                        . . . . . . . . . . ERROR DEF
      WORD7(1)=RELUP*ABS(AMIN)
      UP = WORD7(1)
      IF (UP .LE. 0.)  UP = 1.0
      IF (ISW(2) .GE. 1)  CALL MAPRIN(1,AMIN)
      WORD7(1)=MAXFIN
      WORD7(2)=RELERR*UP
      NFCNMX = WORD7(1) + 0.5
      IF (NFCNMX .LE. 0)  NFCNMX = 1000
      EPSI = WORD7(2)
      IF (EPSI .LE. 0.)  EPSI = 0.1 * UP
      NEWMIN = 0
      ITAUR = 0
      ISW(1) = 0
      CALL MASIMP
      IF(ABS(DIRIN(1)).LE.ABS(EPSMAC*X(1)).AND.
     &   ABS(DIRIN(2)).LE.ABS(EPSMAC*X(2))) THEN
        IF(LST3.GE.1) WRITE(6,2100)
        GOTO 500
      ENDIF
      WORD7(1)=MAXFIN
      RELERR=RELER2*RELERR
      WORD7(2)=RELERR*UP
      NFCNMX = WORD7(1) + 0.5
      IF (NFCNMX .LE. 0)  NFCNMX = 1000
      EPSI = WORD7(2)
      IF (EPSI .LE. 0.)  EPSI = 0.1 * UP
      CALL MASIMP
  500 FCNMAX=ABS(AMIN)
      IF(ISW(1).GE.1) THEN
        IF(LST3.GE.1) WRITE(6,2200)
        FCNMAX=FCNMAX*1.25
      ENDIF
      FMAX=ABS(AMIN)
C                                        . . . . . . . . . . END, EXIT
      WORD7(1)=0.
 1100 IT = WORD7(1) + 0.5
      IF (FVAL3 .EQ. AMIN .OR. IT .GT. 0) RETURN
      IFLAG = 3
      CALL MASIGX(NPAR,GIN,F,U,IFLAG)
      NFCN = NFCN + 1
      IF(LST3.GE.1.AND.ABS(F).GT.FMAX) WRITE(6,2300) F
      RETURN

 2100 FORMAT(' Warning: Stepsizes are less than machine accuracy ',
     &'times variable values. No further minimization attempted.')
 2200 FORMAT(' Warning: Simplex minimization has not converged ',
     &'properly.',/,10X,'Returned maximum increased by a factor 1.25.')
 2300 FORMAT(' Warning from MACMND: function at minimum, ',E12.4,
     &'  is smaller than stored minimum.')

      END
      SUBROUTINE MAIDAT

C...This is the MINUIT routine MIDATA.
CC        GETS PARAMETERS FROM /MAINUI/  AND /MAINUC/
CC        AND SETS UP THE STARTING PARAMETER LISTS.
CC        CONTROL THEN PASSES TO MACMND FOR READING THE COMMAND "CARDS".
CC

      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC
      COMMON /MAINUI/ XKIN(4),UKIN(4),WKIN(4),AIN(4),BIN(4),
     &MAXFIN,RELUP,RELERR,RELER2,FCNMAX
      COMMON /MAINUC/ NAMKIN(4),NAM(30)
      COMMON /LPFLAG/ LST3
      CHARACTER*10 NAMKIN,NAM,NAMK,BLANK
      CHARACTER XTITLE*60
      REAL MMPINT
      DATA BLANK/'          '/
      DATA XTITLE/' FIND MINIMUM OF -(DIFFERENTIAL CROSS SECTION)'/
      DATA MNINIT/0/,IFATAL,NINT/0,0/
C                                        . INITIALIZE NEW DATA BLOCK . .
      IF (MNINIT .EQ. 0)  NBLOCK=0
      MNINIT = 1
      NBLOCK = NBLOCK + 1
      VERSN = 11.79
      IF(LST3.GE.5) THEN
        WRITE (ISYSWR,1004)  MAXINT,MAXEXT,VERSN,NBLOCK
        WRITE (ISYSWR,1005)
      ENDIF
      DO 50 I= 1, 7
   50 ISW(I) = 0
      SIGMA = 0.
      IF(LST3.GE.5) THEN
        WRITE (ISYSWR,1110)  XTITLE,TIME,EPSMAC
        WRITE (ISYSWR,1005)
      ENDIF
      NPFIX = 0
      NINT = 0
      NU = 0
      NPAR = 0
      IFATAL = 0
      IF(LST3.GE.5) WRITE (ISYSWR,1005)
      DO 100 I= 1, MAXEXT
      U(I) = 0.0
      NAM(I) = BLANK
      ERP(I) = 0.0
      ERN(I) = 0.0
      LCODE(I) = 0
  100 LCORSP (I) = 0
      UP = 1.0
      ISW(5) = 1
      IUNIT = ISYSRD
C                                        . . . READ PARAMETER CARDS . .
      ENTRY MAIDA2
      DO 200 I= 1, 200
      IF(I.GE.5) GOTO 250
      XK=XKIN(I)
      NAMK=NAMKIN(I)
      UK=UKIN(I)
      WK=WKIN(I)
      A=AIN(I)
      B=BIN(I)
      K = XK + 0.1
      NU = MAX0(NU,K)
      IF (K .LE. 0)  GO TO 250
      IF (K .LE. MAXEXT)  GO TO 115
      IFATAL = IFATAL + 1
      IF(LST3.GE.1) THEN
        WRITE (ISYSWR,1009)  K,MAXEXT
        WRITE (ISYSWR,1002)  K,NAMK,UK,WK,A,B
      ENDIF
      GO TO 200
  115 CONTINUE
      IF(NAM(K).EQ.BLANK) GO TO 120
C         PREVIOUSLY DEFINED PARAMETER IS BEING REDEFINED
      IF(LST3.GE.1) WRITE(ISYSWR,1007)
      IF(WERR(K).GT..0) NINT=NINT-1
  120 CONTINUE
      NAM(K) = NAMK
      U(K) = UK
      WERR(K) = WK
      IF (WK .GT. 0.0)  GO TO 122
C                                        . . . FIXED PARAMETER . . . .
      IF(LST3.GE.5) WRITE (ISYSWR, 1002)  K,NAMK,UK
      LCODE(K) = 0
      GO TO 160
C                                        . . . VARIABLE PARAMETER . . .
  122 IF(LST3.GE.5) WRITE (ISYSWR, 1002)  K,NAMK,UK,WK,A,B
      NINT = NINT + 1
      ISW(2) = 0
      IF  (A)  140,130,140
  130 IF  (B)  140,135,140
  135 LCODE(K) = 1
      GO TO 160
  140 IF (B-A)  145,142,150
  142 IFATAL = IFATAL + 1
      IF(LST3.GE.1) WRITE (ISYSWR,1010)
      GO TO 150
  145 SAV = B
      B = A
      A = SAV
      IF(LST3.GE.1) WRITE (ISYSWR,1003)
  150 ALIM(K) = A
      BLIM(K) = B
      LCODE(K) = 4
      IF ((B-U(K))*(U(K)-A))  153,155,160
  153 IFATAL = IFATAL + 1
      IF(LST3.GE.1) WRITE (ISYSWR,1011)
      GO TO 160
  155 IF(LST3.GE.1) WRITE (ISYSWR,1006)
  160 CONTINUE
  200 CONTINUE
      IFATAL = IFATAL + 1
      IF(LST3.GE.1) WRITE (ISYSWR,1012)
C                                       . . . END PARAMETER CARDS
C                                       . .   . STOP IF FATAL ERROR
  250 IF(LST3.GE.5) WRITE (ISYSWR,1005)
      IF (NINT .LE. MAXINT)  GO TO 253
      IF(LST3.GE.1) WRITE (ISYSWR,1008)  NINT,MAXINT
      IFATAL = IFATAL + 1
  253 IF (IFATAL .LE. 0)  GO TO 280
      IF(LST3.GE.1) WRITE (ISYSWR,1013)  IFATAL
      IF(LST3.GE.2) STOP
C                                       CALCULATE STEP SIZES DIRIN
  280 NPAR = 0
      DO 300 K= 1, NU
      IF (LCODE(K) .LE. 0)  GO TO 300
      NPAR = NPAR + 1
      LCORSP(K) = NPAR
      SAV = U(K)
      X(NPAR) = MMPINT(SAV,K)
      XT(NPAR) = X(NPAR)
      SAV2 = SAV + WERR(K)
      VPLU = MMPINT(SAV2,K) - X(NPAR)
      SAV2 = SAV - WERR(K)
      VMINU = MMPINT(SAV2,K) - X(NPAR)
      DIRIN(NPAR) = 0.5 * (ABS(VPLU) +ABS(VMINU))
      G2(NPAR) = 2.0 / DIRIN(NPAR)**2
      GSTEP(NPAR) = DIRIN(NPAR)
      IF (LCODE(K) .GT. 1)  GSTEP(NPAR) = -GSTEP(NPAR)
  300 CONTINUE
      SIGMA = 1.0E10
      IUNIT = ISYSRD
      RETURN
C...     THE FORMAT BELOW IS MACHINE-DEPENDENT. (A10) , (A6,4X) , ETC.
 1002 FORMAT (I10,2X,A10,2X,2G12.6,2X,2G12.6)
 1003 FORMAT(' WARNING           - ABOVE LIMITS HAVE BEEN REVERSED.')
 1004 FORMAT (1H1/42X,21(1H*)/42X,21H*   D506   MINUIT   */42X,
     112H* DIMENSIONS, I3, 1H/, I3, 2H */   42X,
     1'*  MODIFICATION OF  *',/,42X,
     111H*   VERSION ,F6.2,4H   */42X,16H* DATA BLOCK NO. ,I3,2H *)
 1005 FORMAT (4X,96(1H*))
 1006 FORMAT(' WARNING           - ABOVE PARAMETER IS AT LIMIT ')
 1007 FORMAT(' WARNING  *******  - PARAMETER REQUESTED ON FOLLOWING',
     1' CARD HAS ALREADY APPEARED.  PREVIOUS VALUES IGNORED.')
 1008 FORMAT('0   TOO MANY VARIABLE PARAMETERS.  YOU REQUEST',I5/,
     +'   THIS VERSION OF MINUIT IS ONLY DIMENSIONED FOR',I4//)
 1009 FORMAT('0FATAL ERROR. PARAMETER NUMBER',I11,' GREATER THAN ',
     +'ALLOWED MAXIMUM',I4)
 1010 FORMAT(' FATAL ERROR. UPPER AND LOWER LIMITS ARE EQUAL.')
 1011 FORMAT(' FATAL ERROR. PARAMETER OUTSIDE LIMITS',/)
 1012 FORMAT('0FATAL ERROR. MORE THAN 200 PARAMETER CARDS',/)
 1013 FORMAT(/I5,' FATAL ERRORS ON PARAMETER CARDS.  ABORT.',//)
 1110 FORMAT(5X,A60,5X,'TIME',F8.3,' SECONDS',/,70X,'MACH. PREC.=',
     &E10.2)
      END
      SUBROUTINE MAINEW

C...This is the MINUIT routine MINNEW.
CC        THIS IS THE MAIN PROGRAM, DISGUISED AS A SUBROUTINE FOR
CC        REASONS OF COMPATIBILITY BETWEEN SYSTEMS.    IT INITIALIZES
CC        SOME CONSTANTS IN COMMON (INCLUDING THE LOGICAL I/O UNIT NOS.)
CC        THEN VERIFIES THAT FCN GIVES THE SAME VALUE WHEN CALLED
CC        TWICE WITH THE SAME ARGUMENTS, AND PASSES CONTROL TO MACMND.
CC

      COMMON /LPFLAG/ LST3
      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC

C         UNIT NUMBERS FOR CARD READER, PRINTER, PUNCH
C
      ISYSRD = 5
      ISYSWR = 6
      ISYSPU = 7
      MAXINT=15
      MAXEXT=30
C                   DETERMINE MACHINE ACCURACY EPSMAC
      EPSMAC = 0.5
      DO 33 I= 1, 100
      EPSMAC = EPSMAC * 0.5
      IF ((1.0+EPSMAC) .EQ. 1.0)  GO TO 35
   33 CONTINUE
      EPSMAC = 1.0E-6
   35 EPSMAC = 2.0 * EPSMAC
C                             . . . . . . . . .
  110 CONTINUE
      NFCN = 1
      CALL MAIDAT
      CALL MAINTO(X)
      IF(LST3.GE.5) WRITE (ISYSWR,120)
  120 FORMAT (/,'0FIRST ENTRY TO FCN ')
      CALL MASIGX(NPAR,GIN,AMIN,U,1)
      CALL MASIGX(NPAR,GIN,AMIN,U,4)
      CALL MAPRIN(1,AMIN)
      CALL MASIGX(NPAR,GIN,F   ,U,4)
      IF  (F .NE. AMIN)  GO TO 160
      NFCN = 3
      CALL MACMND
      RETURN
  160 CONTINUE
      IF(LST3.GE.1) WRITE (ISYSWR,880) AMIN, F
      IF(LST3.GE.2) STOP
  880 FORMAT('0FOR THE ABOVE VALUES OF THE PARAMETERS, FCN IS TIME-',
     +'DEPENDENT',/,'0F = ',E22.14,' FOR FIRST CALL',/,' F =',E22.14,
     +' FOR SECOND')
      END
      SUBROUTINE MAINTO(PINT)

C...This is the MINUIT routine INTOEX.
CC        TRANSFORMS FROM INTERNAL COORDINATES (PINT) TO EXTERNAL
CC        PARAMETERS (U).   THE MINIMIZING ROUTINES WHICH WORK IN
CC        INTERNAL COORDINATES CALL THIS ROUTINE BEFORE CALLING FCN.

      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC

      DIMENSION PINT(2)
      DO 100 I= 1, NU
      J = LCORSP(I)
      IF ( J )  100,100,50
   50 CONTINUE
      IF (LCODE(I) .EQ. 1)  GO TO 80
      AL = ALIM(I)
      U(I) = AL + 0.5 *(SIN(PINT(J)) +1.0) * (BLIM(I) -AL)
      GO TO 100
   80 U(I) = PINT(J)
  100 CONTINUE
      RETURN
      END
      REAL FUNCTION MMPINT(PEXTI,I)

C...This is the MINUIT routine PINTF.
CC        CALCULATES THE INTERNAL PARAMETER VALUE MMPINT CORRESPONDING
CC        TO THE EXTERNAL VALUE PEXTI FOR PARAMETER I.
CC
      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC
      COMMON /LPFLAG/ LST3
      DATA BIG, SMALL / 1.570796326795 , -1.570796326795 /
      IGO = LCODE(I)
      GO TO (100,200,300,400),IGO
C--       IGO = 1  MEANS NO LIMITS
  100 MMPINT = PEXTI
      GO TO 800
  200 CONTINUE
  300 CONTINUE
C--       IGO = 4  MEANS THERE ARE TWO LIMITS
  400 ALIMI = ALIM(I)
      BLIMI = BLIM(I)
      IF (PEXTI-ALIMI)  440,500,460
  440 A = SMALL
  450 MMPINT = A
      PEXTI = ALIMI + 0.5* (BLIMI-ALIMI) *(SIN(A) +1.0)
      LIMSET=1
      IF(LST3.GE.1) WRITE (ISYSWR,241)  I
      GO TO 800
  460 IF (BLIMI-PEXTI)  470,520,480
  470 A = BIG
      GO TO 450
  480 YY=2.0*(PEXTI-ALIMI)/(BLIMI-ALIMI) - 1.0
      MMPINT = ATAN(YY/SQRT(1.0- YY**2) )
      GO TO 800
  500 MMPINT = SMALL
      GO TO 800
  520 MMPINT = BIG
  800 RETURN
  241 FORMAT(' WARNING - VARIABLE',I3,' HAS BEEN BROUGHT BACK IN',
     +'SIDE LIMITS BY MMPINT.')
      END
      SUBROUTINE MAPRIN  (IKODE,FVAL)

C...This is the MINUIT routine MPRINT.
CC        PRINTS THE VALUES OF THE PARAMETERS AT THE TIME OF THE CALL.
CC        ALSO PRINTS OTHER RELEVANT INFORMATION SUCH AS FUNCTION VALUE,
CC        ESTIMATED DISTANCE TO MINIMUM, PARAMETER ERRORS, STEP SIZES.
CC        ACCORDING TO THE VALUE OF IKODE,THE PRINTOUT IS LONG FORMAT,
CC        SHORT FORMAT, OR MINOS FORMAT (0,1,2)
CC

      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC
      COMMON /MAINUC/ NAMKIN(4),NAM(30)
      COMMON /LPFLAG/ LST3
      CHARACTER*10 NAMKIN,NAM
C                                        . GET TIME AND PRINT HEADINGS .
      IF(LST3.GE.5) WRITE (ISYSWR,1000)
      E = SIGMA
      KOUNT = 0
C                                        . . . LOOP OVER PARAMETERS . .
      DO 200 I= 1, NU
      IF(NAM(I).EQ.'          ') GOTO 200
   20 L = LCORSP(I)
      IF (L .EQ. 0)  GO TO 55
C              VARIABLE PARAMETER.  CALCULATE EXTERNAL ERROR IF V EXISTS
      IF (ISW(2) .LT. 1)  GO TO 27
      DX = SQRT(ABS(V(L,L)*UP))
      IF (LCODE(I) .LE. 1)  GO TO 26
      AL = ALIM(I)
      BA = BLIM(I) - AL
      DU1 = AL + 0.5 *(SIN(X(L)+DX) +1.0) * BA - U(I)
      DU2 = AL + 0.5 *(SIN(X(L)-DX) +1.0) * BA - U(I)
      IF (DX .GT. 1.0)  DU1 = BA
      DX = 0.5 * (ABS(DU1) + ABS(DU2))
   26 WERR(I) = DX
   27 X1 = X(L)
      X2 = DIRIN(L)
      IF (IKODE .LT. 2)  GO TO 29
      X1 = ERP(I)
      X2 = ERN(I)
   29 IF (KOUNT)  30,30,40
   30 KOUNT = 1
      IF(LST3.GE.5)
     &WRITE (ISYSWR,1001) FVAL,NFCN,TI,E, L,I,NAM(I),U(I),WERR(I),X1,X2
      GO TO 45
   40 IF(LST3.GE.5) WRITE (ISYSWR,1002)   L,I,NAM(I),U(I),WERR(I),X1,X2
   45 IF (LCODE(I) .LE. 1)  GO TO 200
      IF(LST3.GE.1.AND. ABS(COS(X(L))) .LT. 0.001)  WRITE (ISYSWR,1004)
      GO TO 200
C                           FIXED PARAMETER.  PRINT ONLY IF IKODE .GT.0
   55 IF (IKODE .EQ. 0)  GO TO 200
      IF (KOUNT)  60,60,70
   60 KOUNT = 1
      IF(LST3.GE.5) WRITE (ISYSWR,1001) FVAL,NFCN,TI,E, L,I,NAM(I),U(I)
      GO TO 200
   70 IF(LST3.GE.5) WRITE (ISYSWR,1003)                   I,NAM(I),U(I)
  200 CONTINUE
      IF(LST3.GE.5.AND.
     &IKODE.GE.1 .AND.ISW(2).GE.1)  WRITE (ISYSWR,1005)  UP
      RETURN
 1000 FORMAT(/ 4X,'FCN VALUE',5X,'CALLS',4X,'TIME',4X,' EDM  ',4X ,
     +'INT.EXT. PARAMETER     VALUE         ERROR      INTERN.VALUE  ',
     +'INT.STEP SIZE')
 1001 FORMAT(E15.7,I7,F9.2,E11.2,I6,I4,1X,A10,4E14.5)
 1002 FORMAT(1H ,41X,I6,I4,1X,A10,4E14.5)
 1003 FORMAT(1H  ,47X  ,I4,1X,A10,4E14.5)
 1004 FORMAT(1H ,52X  ,'WARNING -   - ABOVE PARAMETER IS AT LIMIT.')
 1005 FORMAT(/45X,'ERRORS CORRESPOND TO FUNCTION CHANGE OF ',E12.4)
      END
      SUBROUTINE MARAZZ(YNEW,PNEW)

C...This is the MINUIT routine RAZZIA.
CC        CALLED ONLY BY SIMPLEX (AND IMPROV) TO ADD A NEW POINT
CC        AND REMOVE AN OLD ONE FROM THE CURRENT SIMPLEX, AND GET THE
CC        ESTIMATED DISTANCE TO MINIMUM.
CC
      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC
      COMMON /LPFLAG/ LST3
      DIMENSION PNEW(15)
      DO 10 I=1,NPAR
10    P(I,JH)=PNEW(I)
      Y(JH)=YNEW
      IF(YNEW.GE.AMIN) GO TO 18
      DO 15 I=1,NPAR
15    X(I)=PNEW(I)
      CALL MAINTO(X)
      AMIN=YNEW
      JL=JH
18    CONTINUE
      JH=1
      NPARP1=NPAR+1
20    DO 25 J=2,NPARP1
      IF (Y(J) .GT. Y(JH))  JH = J
25    CONTINUE
      SIGMA = Y(JH) - Y(JL)
      IF (SIGMA .LE. 0.)  GO TO 45
      US = 1.0/SIGMA
      DO 35 I= 1, NPAR
      PBIG = P(I,1)
      PLIT = PBIG
      DO 30 J= 2, NPARP1
      IF (P(I,J) .GT. PBIG)  PBIG = P(I,J)
      IF (P(I,J) .LT. PLIT)  PLIT = P(I,J)
   30 CONTINUE
      DIRIN(I) = PBIG - PLIT
      IF (ITAUR .LT. 1 )  V(I,I) = 0.5*(V(I,I) +US*DIRIN(I)**2)
   35 CONTINUE
   40 RETURN
   45 IF(LST3.GE.1) WRITE (ISYSWR, 1000)  NPAR
      GO TO 40
 1000 FORMAT('0***** FUNCTION VALUE DOES NOT SEEM TO DEPEND ON ANY ',
     +'OF THE',I3,' VARIABLE PARAMETERS',/15X ,'VERIFY THAT STEP SIZES',
     +' ARE BIG ENOUGH AND CHECK FCN LOGIC.',/1X,81(1H*)/1X,81(1H*)//)
      END
      SUBROUTINE MASIMP

C...This is the MINUIT routine SIMPLEX.
CC        PERFORMS A MINIMIZATION USING THE SIMPLEX METHOD OF NELDER
CC        AND MEAD (REF. -- COMP. J. 7,308 (1965)).
      COMMON /MAINUI/ XKIN(4),UKIN(4),WKIN(4),AIN(4),BIN(4),
     &MAXFIN,RELUP,RELERR,RELER2,FCNMAX
      COMMON /LPFLAG/ LST3
      COMMON
     1/MAMINE/ ERP(30)  ,ERN(30)
     2/MAPARI/ X(15)    ,XT(15)   ,DIRIN(15) ,MAXINT     ,NPAR
     3/MAPARE/ U(30)              ,WERR(30)  ,MAXEXT     ,NU
     4/MALIMI/ ALIM(30) ,BLIM(30) ,LCODE(30) ,LCORSP(30) ,LIMSET
     5/MAVARI/ V(15,15)
     7/MAFIX / IPFIX(15),XS(15)   ,XTS(15)   ,DIRINS(15) ,NPFIX
     7/MAFIX2/ GRDS(15) ,G2S(15)  ,GSTEPS(15),ABERFS(15)
     C/MACASC/ Y(16)    ,JH       ,JL
     F/MADERI/ GIN(30)  ,GRD(15)  ,G2(15)    ,GSTEP(15)  ,ABERF(15)
     G/MASIMV/ P(15,16) ,PSTAR(15),PSTST(15) ,PBAR(15)   ,PRHO(15)
     J/MAVART/ VT(15,15)
      COMMON
     6/MAUNIT/ ISYSRD   ,ISYSWR   ,ISYSPU
     8/MATITL/ TITLE(13),DATE(2)  ,ISW(7)    ,NBLOCK
     9/MACONV/ EPSI     ,APSI     ,VTEST     ,NSTEPQ     ,NFCN ,NFCNMX
     A/MACARD/ CWORD    ,CWORD2   ,CWORD3    ,WORD7(7)
     B/MAMINI/ AMIN     ,UP       ,NEWMIN    ,ITAUR      ,SIGMA,EPSMAC

      DATA ALPHA,BETA,GAMMA,RHOMIN,RHOMAX / 1.0, 0.5, 2.0, 4.0, 8.0/
      IF (NPAR .LE. 0)  RETURN
      NPFN=NFCN
      NPARP1=NPAR+1
      RHO1 = 1.0 + ALPHA
      RHO2 = RHO1 + ALPHA*GAMMA
      WG = 1.0/FLOAT(NPAR)
      IFLAG=4
      IF(LST3.GE.5) WRITE(ISYSWR,100) EPSI
      DO 2 I= 1, NPAR
      IF (ISW(2) .GE. 1)  DIRIN(I) = SQRT(V(I,I)*UP)
      IF (ABS(DIRIN(I)) .LT. 1.0E-10*ABS(X(I))) DIRIN(I)=1.0E-8*X(I)
      IF(ITAUR.LT. 1)  V(I,I) =    DIRIN(I)**2/UP
    2 CONTINUE
      IF (ITAUR .LT. 1)  ISW(2) = 1
C**       CHOOSE THE INITIAL SIMPLEX USING SINGLE-PARAMETER SEARCHES
    1 CONTINUE
      YNPP1 = AMIN
      JL = NPARP1
      Y(NPARP1) = AMIN
      ABSMIN = AMIN
      DO 10 I= 1, NPAR
      AMING = AMIN
      PBAR(I) = X(I)
      BESTX = X(I)
      KG = 0
      NS = 0
      NF = 0
    4 X(I) = BESTX + DIRIN(I)
      CALL MAINTO(X)
      CALL MASIGX(NPAR,GIN, F, U, 4)
      NFCN = NFCN + 1
      IF (F .LE. AMING)  GO TO 6
C         FAILURE
      IF (KG .EQ. 1)  GO TO 8
      KG = -1
      NF = NF + 1
      DIRIN(I) = DIRIN(I) * (-0.4)
      IF (NF .LT. 3)  GO TO 4
      NS = 6
C         SUCCESS
    6 BESTX = X(I)
      DIRIN(I) = DIRIN(I) * 3.0
      AMING = F
      KG = 1
      NS = NS + 1
      IF (NS .LT. 6)  GO TO 4
C         LOCAL MINIMUM FOUND IN ITH DIRECTION
    8 Y(I) = AMING
      IF (AMING .LT. ABSMIN)  JL = I
      IF (AMING .LT. ABSMIN)  ABSMIN = AMING
      X(I) = BESTX
      DO 9 K= 1, NPAR
    9 P(K,I) = X(K)
   10 CONTINUE
      JH = NPARP1
      AMIN=Y(JL)
      CALL MARAZZ(YNPP1,PBAR)
      DO 20 I= 1, NPAR
   20 X(I) = P(I,JL)
      CALL MAINTO(X)
      DO 30 I=1,NPAR
   30 IF(ABS(DIRIN(I)).LE.ABS(EPSMAC*X(I))) DIRIN(I)=4.*EPSMAC*X(I)
      IF (ISW(5) .GE. 1)  CALL MAPRIN(0,AMIN)
      SIGMA = SIGMA * 10.
      SIG2 = SIGMA
      NCYCL=0
C                                        . . . . .  START MAIN LOOP
   50 CONTINUE
C...Change in SIMPLX; error redefined for second call to MASIMP.
      UP=RELUP*ABS(AMIN)
      EPSI=RELERR*UP
      IF (SIG2 .LT. EPSI .AND. SIGMA.LT.EPSI)     GO TO 76
      SIG2 = SIGMA
      IF ((NFCN-NPFN) .GT. NFCNMX)  GO TO 78
C         CALCULATE NEW POINT * BY REFLECTION
      DO 60 I= 1, NPAR
      PB = 0.
      DO 59 J= 1, NPARP1
   59 PB = PB + WG * P(I,J)
      PBAR(I) = PB - WG * P(I,JH)
   60 PSTAR(I)=(1.+ALPHA)*PBAR(I)-ALPHA*P(I,JH)
      CALL MAINTO(PSTAR)
      CALL MASIGX(NPAR,GIN,YSTAR,U,4)
      NFCN=NFCN+1
      IF(YSTAR.GE.AMIN) GO TO 70
C         POINT * BETTER THAN JL, CALCULATE NEW POINT **
      DO 61 I=1,NPAR
   61 PSTST(I)=GAMMA*PSTAR(I)+(1.-GAMMA)*PBAR(I)
      CALL MAINTO(PSTST)
      CALL MASIGX(NPAR,GIN,YSTST,U,4)
      NFCN=NFCN+1
C         TRY A PARABOLA THROUGH PH, PSTAR, PSTST.  MIN = PRHO
      Y1 = (YSTAR-Y(JH)) * RHO2
      Y2 = (YSTST-Y(JH)) * RHO1
      RHO = 0.5 * (RHO2*Y1 -RHO1*Y2) / (Y1 -Y2)
      IF (RHO .LT. RHOMIN)  GO TO 66
      IF (RHO .GT. RHOMAX)  RHO = RHOMAX
      DO 64 I= 1, NPAR
   64 PRHO(I) = RHO*PBAR(I) + (1.0-RHO)*P(I,JH)
      CALL MAINTO(PRHO)
      CALL MASIGX(NPAR,GIN,YRHO, U,4)
      NFCN = NFCN + 1
      IF (YRHO .LT. Y(JL) .AND. YRHO .LT. YSTST)  GO TO 65
      IF (YSTST .LT. Y(JL))  GO TO 67
      IF (YRHO .GT. Y(JL))  GO TO 66
C         ACCEPT MINIMUM POINT OF PARABOLA, PRHO
   65 CALL MARAZZ (YRHO,PRHO)
      GO TO 68
   66 IF (YSTST .LT. Y(JL))  GO TO 67
      CALL MARAZZ(YSTAR,PSTAR)
      GO TO 68
   67 CALL MARAZZ(YSTST,PSTST)
   68 NCYCL=NCYCL+1
      IF (ISW(5) .LT. 2)  GO TO 50
      IF (ISW(5) .GE. 3 .OR. MOD(NCYCL, 10) .EQ. 0) CALL MAPRIN(0,AMIN)
      GO TO 50
C         POINT * IS NOT AS GOOD AS JL
   70 IF (YSTAR .GE. Y(JH))  GO TO 73
      JHOLD = JH
      CALL MARAZZ(YSTAR,PSTAR)
      IF (JHOLD .NE. JH)  GO TO 50
C         CALCULATE NEW POINT **
   73 DO 74 I=1,NPAR
   74 PSTST(I)=BETA*P(I,JH)+(1.-BETA)*PBAR(I)
      CALL MAINTO (PSTST)
      CALL MASIGX(NPAR,GIN,YSTST,U,4)
      NFCN=NFCN+1
      IF(YSTST.GT.Y(JH)) GO TO 1
C     POINT ** IS BETTER THAN JH
      IF (YSTST .LT. AMIN)  GO TO 67
      CALL MARAZZ(YSTST,PSTST)
      GO TO 50
C                                        . . . . . .  END MAIN LOOP
   76 IF(LST3.GE.5) WRITE(ISYSWR,120)
      GO TO 80
   78 IF(LST3.GE.5) WRITE(ISYSWR,130)
      ISW(1) = 1
   80 DO 82 I=1,NPAR
      PB = 0.
      DO 81 J=1,NPARP1
   81 PB = PB + WG * P(I,J)
   82 PBAR(I) = PB - WG * P(I,JH)
      CALL MAINTO(PBAR)
      CALL MASIGX(NPAR,GIN,YPBAR,U,IFLAG)
      NFCN=NFCN+1
      IF (YPBAR .LT. AMIN)  CALL MARAZZ(YPBAR,PBAR)
      CALL MAINTO(X)
      IF (NFCNMX+NPFN-NFCN .LT. 3*NPAR)  GO TO 90
      IF (SIGMA .GT. 2.0*EPSI)  GO TO 1
   90 CALL MAPRIN(1-ITAUR, AMIN)
      RETURN
  100 FORMAT(' START SIMPLEX MINIMIZATION          ',8X   ,'CON',
     +'VERGENCE CRITERION -- ESTIMATED DISTANCE TO MINIMUM (EDM) .LT.',
     +E10.2 )
120   FORMAT(1H ,'SIMPLEX MINIMIZATION HAS CONVERGED')
130   FORMAT(1H ,'SIMPLEX TERMINATES WITHOUT CONVERGENCE')
      END
